#!/opt/rh/rh-ruby23/root/usr/bin/ruby
# -*- coding: utf-8 -*-

require 'rubygems'
require 'uri'
require 'open3'
require 'yaml'
require 'active_record'

class MURLManager

  BASE_PATH = '/var/lib/redmine'.freeze
  CONFIG_PATH = (BASE_PATH + '/config/database.yml').freeze

  PASSENGER_CONFIG = '/opt/rh/rh-ruby23/root/usr/local/bin/passenger-config'.freeze

  private

  def initialize(argc = 'murl_manage', argv = [])
    @argc = argc.downcase.to_sym
    @argv = argv
    run
  end

  def usage
    STDERR.puts "usage(show):"
    STDERR.puts " #{@argc} show"
    STDERR.puts "usage(set):"
    STDERR.puts " #{@argc} set http://dummy.example.com/redmine"
    exit 1
  end

  def get_config
    config = YAML.load(File.read(CONFIG_PATH))
    ret = nil
    if config.has_key?('production') then
      ret = config['production']
    end
    unless ret then
      STDERR.puts 'invalid config'
      exit 1
    end
    ret
  end

  def db_con
    config_production = get_config
    begin
      ActiveRecord::Base.establish_connection(config_production)
      eval 'class Setting < ActiveRecord::Base; end'
    rescue PG::ConnectionBad => e
      STDERR.puts 'Database Connection/Access Failed:'
      STDERR.puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          STDERR.puts i
        end
      end
      exit 1
    rescue ActiveRecord::StatementInvalid => e
      if e.cause.is_a?(PG::UndefinedTable) then
        STDERR.puts 'database not initialized'
        exit 1
      else
        STDERR.puts 'Unknown Exception:'
        STDERR.puts e.message
        if e.backtrace then
          e.backtrace.each do |i|
            STDERR.puts i
          end
        end
      end
      exit 1
    rescue Exception => e
      STDERR.puts 'Unknown Exception:'
      STDERR.puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          STDERR.puts i
        end
      end
      exit 1
    end
  end

  def run
    cmd = @argv.shift
    usage unless cmd
    case cmd.downcase.to_sym
    when :show then run_show
    when :set then run_set
    else
      usage
    end
  end

  def run_show
    db_con
    protocol = Setting.where(:name => 'protocol').first
    host_name = Setting.where(:name => 'host_name').first
    if protocol and host_name then
      STDERR.puts "#{protocol.value}://#{host_name.value}"
    elsif !protocol and !host_name then
      STDERR.puts "not configured."
      exit 1
    else
      STDERR.puts "Broken Setting."
      STDERR.puts " protocol=#{protocol.value}"
      STDERR.puts " host_name=#{host_name.value}"
      exit 1
    end
    exit 0
  end

  def run_set
    db_con
    url = @argv.shift
    STDERR.puts url
    begin
      url = URI.parse(url)
      if url then
        if 'http' != url.scheme and 'https' != url.scheme then
          STDERR.puts "Invalid Scheme."
          STDERR.puts " scheme=#{url.scheme}"
          exit 1
        end
        if !url.host or url.host.size == 0 then
          STDERR.puts "Invalid Hostname."
          STDERR.puts " hostnmae=#{url.host}"
          exit 1
        end
        if !url.port or url.port.to_i == 0 or url.port.to_i > 65535 then
          STDERR.puts "Invalid Port number."
          STDERR.puts " port=#{url.port}"
          exit 1
        end
        if !url.path or url.path.size == 0 then
          STDERR.puts "Invalid Path."
          STDERR.puts " path=#{url.path}"
          exit 1
        end
        Setting.transaction do
          protocol = Setting.where(:name => 'protocol').first
          host_name = Setting.where(:name => 'host_name').first
          new_host_name = ''
          new_host_name << url.host
          if !('http' == url.scheme and 80 == url.port.to_i) and
              !('https' == url.scheme and 443 == url.port.to_i) then
            new_host_name << ":#{url.port}"
          end
          if url.path[0] != '/' then
            new_host_name << '/'
          end
          new_host_name << url.path
          if protocol then
            protocol.value = url.scheme
            protocol.save
          else
            protocol = Setting.create(:name => 'procotol', :value => url.scheme)
          end
          if host_name then
            host_name.value = new_host_name
            host_name.save
          else
            host_name = Setting.create(:name => 'host_name', :value => new_host_name)
          end
        end
      end
    rescue URI::InvalidURLError => e
      STDERR.puts "Invalid URL."
      STDERR.puts " url=#{url}"
      exit 1
    rescue Exception => e
      STDERR.puts "Unknown Error."
      STDERR.puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          STDERR.puts i
        end
      end
      exit 1
    end
    STDERR.puts 'Ok'
    exit 0
  end

end

if $0 == __FILE__ then
  id = MURLManager.new($0, ARGV)
end
