# -*- coding: utf-8 -*-

require 'date'
require 'sidfm_controller_support'

class SidfmRuleSettingController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private
  
  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles
  before_action :define_user_custom_category_ids

  before_action :get_vm_version
  before_action :clear_import_session
  before_action :clear_export_session
  before_action -> { get_summary_table_lines('rule') } 

  def check_roles 
    if !@sidfm_roles.include?(:view) then
      redirect_to :action => :index, :notice => af_log_user('許可されていない操作です。') and return
    end
    if !@sidfm_roles.include?(:rule_admin) and
       ![ 'api_get_display_summary_content', 'api_get_display_summary_rows', 'index' ].include?(action_name)
    then
      redirect_to :action => :index, :notice => af_log_user('許可されていない操作です。') and return
    end
  end

  def define_user_custom_category_ids 
    @user_custom_category_ids = if @sidfm_roles.include?(:admin) then
                                  SidfmdCategory.where(:is_system => false)
                                                .pluck(:id)
                                else
                                  User.current
                                      .categories
                                      .pluck(:id)
                                end
  end

  # メール送信の操作の送信/送信先チェック
  # カテゴリが「全体」なら管理者かカテゴリのあるユーザのみ
  # そうでないならカテゴリに属するユーザのみ
  def get_filtered_mail_user_ids(category_id, user_ids)
    user_ids = Array.wrap(user_ids)
    filtered_user_ids = (category_id.to_i == 0 ?
      User.where("id IN (?) OR id IN (?)",
        User.get_category_users(SidfmdCategory.pluck(:id)).select(:id),
        Member.joins(:roles)
              .where(:roles => { :name => ROLE_SIDFM_ADM })
              .where(:members => { :project_id => @sidfm_project.id })
              .select(:user_id)
      ).where(:id => user_ids) :
      User.get_category_users([ category_id.to_i ])
          .where(:id => user_ids)
    ).pluck(:id)
  end

  def create_rule_param(rule_name, rule_category, rule_position, is_enable, rule_trigger, rule_conditions, rule_actions, updated_at)
    trigger_name = rule_trigger.class.table_name
    send_mail_user_ids = []
    send_mail_user_emails = []
    send_mail_user_record = []
    param = {}
    param["rule_updated_at"] = updated_at
    param["rule_name"] = rule_name
    param["rule_position"] = rule_position.to_i
    param["is_enable"] = is_enable
    param["category"] = rule_category ? rule_category : "all"
    param["trigger"] = if trigger_name == "sidfmd_rule_tp_hours" then
                         {
                           "hostname" => "",
                           "period" => "periodic-hour",
                           "value" => rule_trigger.hour,
                           "unit" => "",
                           "category" => "",
                         }
                       elsif trigger_name == "sidfmd_rule_tp_intervals" then
                         {
                           "hostname" => "",
                           "period" => "periodic-interval",
                           "value" => rule_trigger.value,
                           "unit" => rule_trigger.unit,
                           "category" => "",
                         }
                       elsif trigger_name == "sidfmd_rule_t_vulsyncs" then
                         {
                           "hostname" => rule_trigger.sidfmd_host_id ? SidfmdHost.find(rule_trigger.sidfmd_host_id).hostname : "",
                           "period" => "vulsync",
                           "value" => "vulsync",
                           "unit" => "",
                           "category" => rule_trigger.sidfmd_category_id ? rule_trigger.sidfmd_category_id : "all",
                         }
                       end
    cond_param = {}
    ctag_data_id = 0 #カスタムタグのデータを特定するために使用する
    timerange_data_id = 0 #期間指定のデータを特定するために使用する
    rule_conditions.each do |cond|
      cond = cond.detail
      cond_name = cond.class.table_name
      if cond_name == "sidfmd_rule_c_issue_updates" then
        if cond.cond_or && cond.cnt_create > 0 then
          cond_param["issue-update-regi"] = cond.cnt_create
        elsif cond.cond_or && cond.cnt_remove > 0 then
          cond_param["issue-update-del"] = cond.cnt_remove
        elsif !cond.cond_or then
          cond_param["issue-update-regi-and-del"] = {"regi" => cond.cnt_create, "del" => cond.cnt_remove}
        end
      elsif cond_name == "sidfmd_rule_c_sri_aboves" then
        cond_param["sri-above"] = cond.risk_id
      elsif cond_name == "sidfmd_rule_c_sri_belows" then
        cond_param["sri-below"] = cond.risk_id
      elsif cond_name == "sidfmd_rule_c_cvss_aboves" then
        cond_param["cvss-above"] = {
          "val" => cond.cvss_value,
          "include_notreg" => cond.is_notreg.to_s,
          "flg_exploitable" => if cond.exploitable.nil? then
                                 "all"
                               elsif cond.exploitable == false then
                                 "not_exist"
                               else
                                 "exist"
                               end
        }
      elsif cond_name == "sidfmd_rule_c_cvss_belows" then
        cond_param["cvss-below"] = {
          "val" => cond.cvss_value,
          "include_notreg" => cond.is_notreg.to_s,
          "flg_exploitable" => if cond.exploitable.nil? then
                                 "all"
                               elsif cond.exploitable == false then
                                 "not_exist"
                               else
                                 "exist"
                               end
      }
      elsif cond_name == "sidfmd_rule_c_cvss_notregs" then
        cond_param["cvss-notreg"] = cond.is_notreg.to_s
      elsif cond_name == "sidfmd_rule_c_warnings" then
        if cond.is_cvss then
          cond_param["cvss-warning"] = cond.is_warning.to_s
        else
          cond_param["sri-warning"] = cond.is_warning.to_s
        end
      elsif cond_name == "sidfmd_rule_c_cvss_vectors" then
        cond_param["cvss-vector"] = {
          "vectors" => {
            "AV" => cond.av ? cond.av.split('/') : [ 'NONE' ],
            "AC" => cond.ac ? cond.ac.split('/') : [ 'NONE' ],
            "PR" => cond.pr ? cond.pr.split('/') : [ 'NONE' ],
            "UI" => cond.ui ? cond.ui.split('/') : [ 'NONE' ],
            "S"  => cond.s ? cond.s.split('/') : [ 'NONE' ],
            "C" => cond.c ? cond.c.split('/') : [ 'NONE' ],
            "I" => cond.i ? cond.i.split('/') : [ 'NONE' ],
            "A" => cond.a ? cond.a.split('/') : [ 'NONE' ]
          },
          "flg_perfect" => cond.flg_perfect.to_s,
          "flg_ignore_undef" => cond.flg_ignore_undef.to_s
        }
      elsif cond_name == "sidfmd_rule_c_history_posts" then
        cond_param["history-create"] = { "val" => cond.cnt }
        users = cond.sidfmd_rule_chp_user_subs
        if users.exists? then
          cond_param["history-create"]["flg_exclusion"] = cond.flg_exclusion.to_s
          cond_param["history-create"]["user_ids"] = users.pluck(:user_id) 
        end
      elsif cond_name == "sidfmd_rule_c_duedate_expireds" then
        cond_param["due-date-expired"] = "due-date-expired"
      elsif cond_name == "sidfmd_rule_c_duedate_undefs" then
        cond_param["due-date-undef"] = "due-date-undef"
      elsif cond_name == "sidfmd_rule_c_duedate_approachings" then
        cond_param["due-date-approach"] = { "val" => cond.days }
      elsif cond_name == "sidfmd_rule_c_match_categories" then
        sub_categories = cond.sidfmd_rule_cm_category_subs.map{|c| c.sidfmd_category_id}
        if cond.inv then
          cond_param["category-unmatch"] = sub_categories
        else
          cond_param["category-match"] = sub_categories
       end
      elsif cond_name == "sidfmd_rule_c_match_hosts" then
        host_ids = cond.sidfmd_rule_cm_host_subs.map{|h| h.sidfmd_host_id}
        hosts = SidfmdHost.where(:id => host_ids)
        hostnames = hosts.map{|h| h.hostname}
        if cond.inv then
          cond_param["host-unmatch"] = hostnames
        else
          cond_param["host-match"] = hostnames
        end
      elsif cond_name == "sidfmd_rule_c_host_syncings" then
        cond_param["host-syncing"] = cond.inv ? 'false' : 'true'
      elsif cond_name == "sidfmd_rule_c_statuses" then
        status_ids = [ cond.status_id1 ]
        status_ids.push(cond.status_id2) if cond.status_id2
        cond_param["status-match"] = status_ids
      elsif cond_name == "sidfmd_rule_c_match_apps" then
        sub_apps = cond.sidfmd_rule_cm_app_subs
        sub_lapps = cond.sidfmd_rule_cm_lapp_subs
        prod_ids = []
        ver_ids = []
        lprod_ids = []
        lver_ids = []
        sub_apps.each do |app|
          if !app.version_id then
            prod_ids.push(app.product_id)
          else
            ver_ids.push(app.version_id)
          end
        end
        sub_lapps.each do |lapp|
          if !lapp.sidfmd_lversion_id then
            lprod_ids.push(lapp.sidfmd_lproduct_id)
          else
            lver_ids.push(lapp.sidfmd_lversion_id)
          end
        end
        match_app_param = {
          "product-ids" => prod_ids,
          "version-ids" => ver_ids,
          "lproduct-ids" => lprod_ids,
          "lversion-ids" => lver_ids
        }
        if cond.inv then
          cond_param["product-unmatch"] = match_app_param
        else
          cond_param["product-match"] = match_app_param
        end
      elsif cond_name == "sidfmd_rule_c_cvss_exploitables" then
        cond_param["cvss-exploitable"] = {
          "value" => cond.value.to_s,
          "is_change" => cond.is_change.to_s
        }
      elsif cond_name == "sidfmd_rule_c_ctags" then
        cond_param["ctag"] = [] unless cond_param["ctag"]
        enable_ctags = []
        not_enable_ctags = []
        cond.sidfmd_rule_cm_ctags.joins("
          INNER JOIN sidfmd_ctags ON sidfmd_ctags.id = sidfmd_rule_cm_ctags.sidfmd_ctag_id
        ")
        .joins(%|
          LEFT OUTER JOIN sidfmd_categories ON sidfmd_categories.id = sidfmd_ctags.sidfmd_category_id
        |)
        .select(
          "sidfmd_ctags.id",
          "sidfmd_ctags.keyword AS name",
          "sidfmd_ctags.color",
          "sidfmd_ctags.bgcolor",
          "sidfmd_ctags.priority",
          "sidfmd_categories.name AS category_name",
          :is_enable
        )
        .order("sidfmd_ctags.priority DESC")
        .each_with_index do |ctag|
          ctag_data = {
            "id" => ctag.id,
            "name" => ctag.name,
            "color" => ctag.color,
            "bgcolor" => ctag.bgcolor,
            "priority" => ctag.priority,
            "category_name" => ctag.category_name,
          }
          if ctag.is_enable then
            enable_ctags << ctag_data
          else
            not_enable_ctags << ctag_data
          end
        end
        cond_param["ctag"] << { "enable" => enable_ctags, "not_enable" => not_enable_ctags, "dataid" => ctag_data_id }
        ctag_data_id += 1;
      elsif cond_name == "sidfmd_rule_c_timeranges" then
        cond_param["timerange"] = [] unless cond_param["timerange"]
        cond_param["timerange"] << {
          "target" => cond.target,
          "unit" =>  cond.unit,
          "value" => cond.value,
          "except" => cond.inv,
          "dataid" => timerange_data_id
        }
        timerange_data_id += 1
      end
    end
    param["condition"] = cond_param

    action_param = {}
    rule_actions.each do |action|
      action_target = action.target.to_s
      action_flg = action.proc_empty.to_s
      action = action.detail
      action_name = action.class.table_name
      if action_name == "sidfmd_rule_a_mail_categories" then
        if action.from_user_id then
          sender_info  = User.find(action.from_user_id)
          sender = { "userid" => action.from_user_id, "username" => sender_info.lastname + sender_info.firstname }
        else
          sender = { "address" => action.from_email }
        end
        action_param["send-mail-category"] = {
          "sendby" => { "ids" => [(action.sidfmd_category_id ? action.sidfmd_category_id : 'all')] },
          "title" => action.subject,
          "content" => action.body,
          "sender" => sender,
          "target" => action_target,
          "proc_empty" => action_flg
        }
      elsif action_name == "sidfmd_rule_a_mail_users" then
        if action.user_id then
          send_mail_user_ids.push(action.user_id)
        else
          send_mail_user_emails.push(action.email)
        end
          send_mail_user_record.push(action) if send_mail_user_record.size == 0
      elsif action_name == "sidfmd_rule_a_status_changes" then
        status_ids_from = [action.prev_status1]
        status_ids_from.push(action.prev_status2) if action.prev_status2
        action_param["status-change"] = {
          "status_ids_from" => status_ids_from,
          "status_ids_to" => [action.status_id],
          "target" => action_target,
          "proc_empty" => action_flg
        }
      elsif action_name == "sidfmd_rule_a_duedate_changes" then
        if action.fixed_date then
          action_param["due-date-change"] = {
            "date" => action.fixed_date.strftime("%Y/%m/%d"),
            "permit-update" => action.permit_update.to_s,
            "target" => action_target,
            "proc_empty" => action_flg
          }
        else
          action_param["due-date-change"] = {
            "days" => action.refdt_days,
            "permit-update" => action.permit_update.to_s,
            "target" => action_target,
            "proc_empty" => action_flg
          }
        end
      elsif action_name == "sidfmd_rule_a_send_syslogs" then
        action_param["send-syslog"] = {
          "protocol" => action.protocol,
          "address" => SidfmdRuleASendSyslog.get_address(action.target),
          "selected_address" => (action.target.include?("#{SidfmdRuleASendSyslog::IP_ADDRESS_IDX}") ? 'ip_address' : 'hostname'),
          "port_number" => action.port.to_s,
          "facility" => action.facility,
          "severity" => action.severity,
          "hostname" => action.hostname,
          "ident" => action.ident,
          "message" => action.message,
          "target" => action_target,
          "proc_empty" => action_flg
        }
      elsif action_name == "sidfmd_rule_a_ctags" then
        am_ctag_enables = []
        am_ctag_not_enables = []
        action.sidfmd_rule_am_ctags.joins("
          INNER JOIN sidfmd_ctags ON sidfmd_ctags.id = sidfmd_rule_am_ctags.sidfmd_ctag_id
        ")
        .joins(%|
          LEFT OUTER JOIN sidfmd_categories ON sidfmd_categories.id = sidfmd_ctags.sidfmd_category_id
        |)
        .select(
          "sidfmd_ctags.id",
          "sidfmd_ctags.keyword AS name",
          "sidfmd_ctags.color",
          "sidfmd_ctags.bgcolor",
          "sidfmd_ctags.priority",
          "sidfmd_categories.name AS category_name",
          :is_enable
        )
        .order("sidfmd_ctags.priority DESC")
        .each do |ctag|
          ctag_data = {
            "id" => ctag.id,
            "name" => ctag.name,
            "color" => ctag.color,
            "bgcolor" => ctag.bgcolor,
            "priority" => ctag.priority,
            "category_name" => ctag.category_name,
          }
          if ctag.is_enable then
            am_ctag_enables << ctag_data
          else
            am_ctag_not_enables << ctag_data
          end
        end
        action_param["set-ctag"] = {
          "enable" => am_ctag_enables,
          "not_enable" => am_ctag_not_enables,
          "target" => action_target,
          "proc_empty" => action_flg
        }
      end
    end
    if send_mail_user_record.size == 1 then
      record = send_mail_user_record[0]
      record_action = record.sidfmd_rule_action
      if send_mail_user_ids.size > 0 then
        sendby = { "ids" => send_mail_user_ids }
      else 
        sendby ={ "address" => send_mail_user_emails }
      end
      if record.from_user_id then
        sender_info = User.find(record.from_user_id)
        sender = { "userid" => record.from_user_id, "username" => sender_info.lastname + sender_info.firstname }
      else
        sender = { "address" => record.from_email }
      end
      action_param["send-mail-user"] = {
        "sendby" => sendby,
        "title" => record.subject,
        "content" => record.body,
        "sender" => sender,
        "target" => record_action.target.to_s,
        "proc_empty" => record_action.proc_empty.to_s
      }
    end
    param["action"]= action_param
    return param
  end

  #プロダクト限定・除外の登録
  def regist_cond_product_match_data(rule_id, product_ids:, version_ids:, lproduct_ids:, lversion_ids:, is_match:)
    product_ids = SidfmdProduct.where(:pid => product_ids).pluck(:pid) #不正なidを排除
    version_ids = SidfmdVersion.where(:vid => version_ids).pluck(:vid) 
    lproduct_ids = SidfmdLproduct.where(:id => lproduct_ids).pluck(:id)
    lversion_ids = SidfmdLversion.where(:id => lversion_ids).pluck(:id)
    if product_ids.size == 0 && version_ids.size == 0 && lproduct_ids.size == 0 &&  lversion_ids.size == 0 then
      raise Exception
    end
    rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
    if rule_cond.save then
      app_match = SidfmdRuleCMatchApp.new(
        sidfmd_rule_cond_id: rule_cond.id,
        inv: !is_match
      )
      if app_match.save then
        product_ids = SidfmdProduct.where(:pid => product_ids).pluck(:pid) #不正なidを排除
        
        product_ids.each do |p_id|
          SidfmdRuleCmAppSub.create(
            sidfmd_rule_c_match_app_id: app_match.id,
            product_id: p_id
          )
        end
        version_ids.each do |v_id|
          SidfmdRuleCmAppSub.create(
            sidfmd_rule_c_match_app_id: app_match.id,
            product_id:  SidfmdProduct.joins(:sidfmd_versions)
                                      .where(:sidfmd_versions => { :vid => v_id })
                                      .first
                                      .pid,
            version_id: v_id
          )
        end
        #ローカルプロダクト
        lproduct_ids.each do |p_id|
          SidfmdRuleCmLappSub.create(
            sidfmd_rule_c_match_app_id: app_match.id,
            sidfmd_lproduct_id: p_id
          )
        end
        lversion_ids.each do |v_id|
          SidfmdRuleCmLappSub.create(
            sidfmd_rule_c_match_app_id: app_match.id,
            sidfmd_lproduct_id: SidfmdLversion.find(v_id).sidfmd_lproduct_id,
            sidfmd_lversion_id: v_id
          )
        end
      end
    end 
  end

  # ログの保持と更新
  # is_updated_status、is_updated_settingsの少なくとも１つはtrueである必要がある
  def create_updated_rule_logs(rule, prev_trigger: nil, prev_conds: nil, prev_actions: nil, is_updated_status:, is_updated_settings:)
    messages = []
    if is_updated_settings then
      messages << SidfmdRule::LOG_RULE_UPDATE_MSG
    end
    if is_updated_status then
      messages << (rule.flg_enable ? SidfmdRule::LOG_RULE_ENABLE_MSG : SidfmdRule::LOG_RULE_DISABLE_MSG)
    end
    if messages.size > 0 then
      rule.create_updated_rule_logs(
        :prev_trigger => prev_trigger,
        :prev_conds => prev_conds,
        :prev_actions => prev_actions,
        :messages => messages  
      )
    end
  end

  def set_rule(rulename, category, flg, position, trigger, condition, operation, method_name, rule_id, updated_at = '', only_flg_changed = false)
    _prev_trigger = nil # 変更前のタイミング 変更時のみnilでなくなる
    _prev_conds = nil # 変更前の条件 変更時のみnilでなくなる
    _prev_actions = nil # 変更前の操作 変更時のみnilでなくなる
    _log_rule_id = nil #ログを更新するルールのid 変更時のみnilでなくなる
    _is_updated_rule_status = false
    begin
      SidfmdRule.transaction do
        unless @sidfm_roles.include?(:admin) then
          if !category || !@user_custom_category_ids.include?(category.to_i) then
            raise Exception
          end
        end
        _rule_category = category
        if position.to_i > SidfmdRule::MAX_POSITION_NUM or position.to_i < SidfmdRule::MIN_POSITION_NUM then
          logger.error('invalid position')
          raise Exception
        end

        if method_name == 'create' then
          target_rule = SidfmdRule.new(
                          sidfmd_category_id: category,
                          position: position.to_i,
                          flg_enable: flg == "enable" ? true : false,
                          name: rulename,
                        )
          if target_rule.save then
            rule_id = target_rule.id
          end
          rule_trigger = SidfmdRuleTrigger.new(sidfmd_rule_id: rule_id)
        elsif method_name == 'modify' then
          target_rule = SidfmdRule.lock.find(rule_id)
          if target_rule.updated_at >= updated_at then #更新されていないかチェック
            render :layout => false, :json => { :err_message => '別のユーザに更新されています。' } and return 
          end
          if only_flg_changed == 'true' then
            target_rule.update(:flg_enable => flg == "enable" ? true : false)
            begin
              create_updated_rule_logs(target_rule, is_updated_status: true, is_updated_settings: false)
            rescue Exception => e
              log_exception(e)
            end
            return true
          end
          _prev_trigger = target_rule.sidfmd_rule_trigger.destroy
          _prev_conds = target_rule.sidfmd_rule_conds.destroy_all
          _prev_actions = target_rule.sidfmd_rule_actions.destroy_all
          _log_rule_id = rule_id
          _is_updated_rule_status = target_rule.flg_enable != (flg == "enable" ? true : false)
          target_rule.update_attributes(
            sidfmd_category_id: category,
            position: position.to_i,
            flg_enable: flg == "enable" ? true : false,
            name: rulename,
            updated_at: Time.now
          )
          rule_trigger = SidfmdRuleTrigger.new()
          target_rule.sidfmd_rule_trigger = rule_trigger
        end
        if rule_trigger.save then
        #タイミング作成
          period = trigger["period"]
          num_of_unit = trigger["value"]
          unit = trigger["unit"]
          category = trigger["category"]
          if period == "periodic-hour" then
            trigger_setting = SidfmdRuleTpHour.new(
                                                sidfmd_rule_trigger_id: rule_trigger.id,
                                                hour: num_of_unit.to_i
                                              )
          elsif period == "periodic-interval" then
            trigger_setting = SidfmdRuleTpInterval.new(
                                                    sidfmd_rule_trigger_id: rule_trigger.id,
                                                    value: num_of_unit.to_i,
                                                    unit: unit
                                                   )
          elsif period == "vulsync" then
            host = SidfmdHost.find_by(:hostname => trigger["hostname"])
            host_id = host ? host.id : nil
            if host_id then
              if target_rule.sidfmd_category_id then
                if !SidfmdCategoriesHost.where(:sidfmd_host_id => host_id, :sidfmd_category_id => target_rule.sidfmd_category_id).exists? then
                  raise EXception
                end
              elsif category != "all" then
                if !SidfmdCategoriesHost.where(:sidfmd_host_id => host_id, :sidfmd_category_id => category).exists? then
                  raise EXception
                end
              else
                if !SidfmdCategoriesHost.joins(:sidfmd_category)
                                        .where(:sidfmd_categories => { :is_system => false })
                                        .where(:sidfmd_categories_hosts => { :sidfmd_host_id => host_id })
                                        .exists? then
                  raise EXception
                end
              end
            end
            trigger_setting = SidfmdRuleTVulsync.new(
                                                  sidfmd_rule_trigger_id: rule_trigger.id,
                                                  sidfmd_category_id: category == "all" ? nil : category.to_i,
                                                  sidfmd_host_id: host_id
                                                 )
          end
          trigger_setting.save
        end
        #条件作成
        cond_vul_regi = condition["issue-update-regi"]
        cond_vul_del = condition["issue-update-del"]
        cond_vul_regi_and_del = condition["issue-update-regi-and-del"]
        cond_sri_above = condition["sri-above"]
        cond_sri_below = condition["sri-below"]
        cond_sri_warning = condition["sri-warning"]
        cond_cvss_above = condition["cvss-above"]
        cond_cvss_below = condition["cvss-below"]
        cond_cvss_warning = condition["cvss-warning"]
        cond_cvss_notreg = condition["cvss-notreg"]
        cond_cvss_vector = condition["cvss-vector"]
        cond_due_date_expired = condition["due-date-expired"]
        cond_due_date_approach = condition["due-date-approach"]
        cond_due_date_undef = condition["due-date-undef"]
        cond_history_create = condition["history-create"]
        cond_category_match = condition["category-match"]
        cond_category_unmatch = condition["category-unmatch"]
        cond_host_match = condition["host-match"]
        cond_host_unmatch = condition["host-unmatch"]
        cond_host_syncing = condition["host-syncing"]
        cond_status_match = condition["status-match"]
        cond_product_match = condition["product-match"]
        cond_product_unmatch = condition["product-unmatch"]
        cond_cvss_exploitable = condition["cvss-exploitable"]
        cond_ctags = condition["ctag"] #複数のカスタムタグの条件がある可能性あり(配列)
        cond_timeranges = condition["timerange"]

        if cond_vul_regi then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCIssueUpdate.create(
                                    sidfmd_rule_cond_id: rule_cond.id,
                                    cond_or: true,
                                    cnt_create: cond_vul_regi.to_i,
                                    cnt_remove: 0
                                   )
          end
        end
        if cond_vul_del then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCIssueUpdate.create(
                                    sidfmd_rule_cond_id: rule_cond.id,
                                    cond_or: true,
                                    cnt_create: 0,
                                    cnt_remove: cond_vul_del.to_i    
                                   )
          end
        end
        if cond_vul_regi_and_del then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCIssueUpdate.create(
                                   sidfmd_rule_cond_id: rule_cond.id,
                                   cond_or: false,
                                   cnt_create: cond_vul_regi_and_del["regi"].to_i,
                                   cnt_remove: cond_vul_regi_and_del["del"].to_i
                                   )
          end
        end
        if cond_sri_above then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCSriAbove.create(
                                 sidfmd_rule_cond_id: rule_cond.id,
                                 risk_id: cond_sri_above.to_i
                                )
          end
        end
        if cond_sri_below then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCSriBelow.create(
                                sidfmd_rule_cond_id: rule_cond.id,
                                risk_id: cond_sri_below.to_i
                                )
          end
        end
        if cond_sri_warning then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCWarning.create(
                               sidfmd_rule_cond_id: rule_cond.id,
                               is_warning: cond_sri_warning == "true" ? true : false,
                               is_cvss: false
                               )
          end
        end
        if cond_cvss_above then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            flg_exploitable = cond_cvss_above["flg_exploitable"]
            SidfmdRuleCCvssAbove.create(
                                 sidfmd_rule_cond_id: rule_cond.id,
                                 is_notreg: cond_cvss_above["include_notreg"] == "true" ?  true : false,
                                 cvss_value: cond_cvss_above["val"].to_f,
                                 exploitable: if flg_exploitable == "all" then
                                                nil
                                              elsif flg_exploitable == "exist" then
                                                true
                                              else
                                                false
                                              end
                                 )
          end
        end
        if cond_cvss_below then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
              flg_exploitable = cond_cvss_below["flg_exploitable"]
              SidfmdRuleCCvssBelow.create(
                                   sidfmd_rule_cond_id: rule_cond.id,
                                   is_notreg: cond_cvss_below["include_notreg"] == "true" ?  true : false,
                                   cvss_value: cond_cvss_below["val"].to_f,
                                   exploitable: if flg_exploitable == "all" then
                                                  nil
                                                elsif flg_exploitable == "exist" then
                                                  true
                                                else
                                                  false
                                                end
                                 )
          end
        end
        if cond_cvss_warning then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCWarning.create(
                                sidfmd_rule_cond_id: rule_cond.id,
                                is_warning: cond_cvss_warning == "true" ? true : false,
                                is_cvss: true
                               )
          end
        end
        if cond_cvss_notreg then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCCvssNotreg.create(
                                  sidfmd_rule_cond_id: rule_cond.id,
                                  is_notreg: cond_cvss_notreg == "true" ? true : false
                                  )
          end
        end
        if cond_cvss_vector then
          vectors = get_cvss_vectors
          selected_vectors = cond_cvss_vector["vectors"]
          #validation
          if (selected_vectors.keys - vectors.keys).size == 0 then
            is_pass = true
            selected_vectors.each do |item, values|
              if !values or values.empty? then
                is_pass = false
                break
              end
              all_values = vectors[item]['values'].keys
              if (values - all_values).size != 0 or (values.include?('NONE') and values.size > 1) then
                is_pass = false
                break
              end
            end
            if is_pass then
              #register
              rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
              if rule_cond.save then
                SidfmdRuleCCvssVector.create(
                  sidfmd_rule_cond_id: rule_cond.id,
                  av: selected_vectors["AV"].first == 'NONE' ? nil : selected_vectors["AV"].join('/'),
                  ac: selected_vectors["AC"].first == 'NONE' ? nil : selected_vectors["AC"].join('/'),
                  pr: selected_vectors["PR"].first == 'NONE' ? nil : selected_vectors["PR"].join('/'),
                  ui: selected_vectors["UI"].first == 'NONE' ? nil : selected_vectors["UI"].join('/'),
                  s: selected_vectors["S"].first == 'NONE' ? nil : selected_vectors["S"].join('/'),
                  c: selected_vectors["C"].first == 'NONE' ? nil : selected_vectors["C"].join('/'),
                  i: selected_vectors["I"].first == 'NONE' ? nil : selected_vectors["I"].join('/'),
                  a: selected_vectors["A"].first == 'NONE' ? nil : selected_vectors["A"].join('/'),
                  flg_perfect: cond_cvss_vector["flg_perfect"] == "true",
                  flg_ignore_undef: cond_cvss_vector["flg_perfect"] == "true" ? cond_cvss_vector["flg_ignore_undef"] == "true" : false
                )
              end
            else
              raise Exception #普通の操作ではこのスコープには入らない
            end
          end
        end
        if cond_due_date_expired then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCDuedateExpired.create(
                                      sidfmd_rule_cond_id: rule_cond.id,
                                     )
          end
        end
        if cond_due_date_approach then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            days = cond_due_date_approach["val"].to_i
            SidfmdRuleCDuedateApproaching.create(
              sidfmd_rule_cond_id: rule_cond.id,
              days: days #指定した日数以降を条件とする場合は、負の値
            )
          end
        end
        if cond_due_date_undef then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          rule_cond.build_sidfmd_rule_c_duedate_undef
          rule_cond.save
        end
        if cond_history_create then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          cnt = cond_history_create["val"]
          history_cond = SidfmdRuleCHistoryPost.new(
            cnt: cnt.to_i,
            flg_exclusion: (cond_history_create['flg_exclusion'] ? (cond_history_create['flg_exclusion'] == 'true') : false)
          )
          if cond_history_create['flg_exclusion'] then
            f_user_ids = []
            if target_rule.sidfmd_category_id then
              catgory_users = User.get_category_users([target_rule.sidfmd_category_id])
                                  .where(:id => cond_history_create['user_ids'])
              f_user_ids = catgory_users.pluck(:id)
              sidfmadm_user = User.where(:id => cond_history_create['user_ids']).where(:login => 'sidfmadm').first
              f_user_ids << sidfmadm_user.id if sidfmadm_user
            else
              if @sidfm_roles.include?(:admin) then
                f_user_ids = User.where(:id => cond_history_create['user_ids']).pluck(:id)
              else
                #普通に操作すれば、ルール登録中に権限が変わらない限りここには入らない
                raise Exception
              end           
            end
            if f_user_ids.size == 0 then
              #validation
              #カテゴリに属するユーザもsidfmadmユーザも無い場合はエラー
              raise Exception
            end
            f_user_ids.uniq!
            history_cond.sidfmd_rule_chp_user_subs = f_user_ids.map{|user_id| SidfmdRuleChpUserSub.new(user_id: user_id) }
          end
          rule_cond.sidfmd_rule_c_history_post = history_cond
          unless rule_cond.save then
            raise Exception
          end
        end
        if cond_category_match then
          if !@sidfm_roles.include?(:admin) then
            pass = true
            system_cat_ids = SidfmdCategory.system.pluck(:id)
            cond_category_match.each do |c_id|
              if !system_cat_ids.include?(c_id.to_i) and !@user_custom_category_ids.include?(c_id.to_i) then
                pass = false
                break
              end
            end
            raise Exception unless pass
          end
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            c_match = SidfmdRuleCMatchCategory.new(
                                               sidfmd_rule_cond_id: rule_cond.id,
                                               inv: false
                                               )
            if c_match.save then
              cond_category_match.each do |cat_id|
                SidfmdRuleCmCategorySub.create(
                                        sidfmd_rule_c_match_category_id: c_match.id,
                                        sidfmd_category_id: cat_id.to_i
                                       )
              end
            end
          end
        end
        if cond_category_unmatch then
          if !@sidfm_roles.include?(:admin) then
            pass = true
            system_cat_ids = SidfmdCategory.system.pluck(:id)
            cond_category_unmatch.each do |c_id|
              if !system_cat_ids.include?(c_id.to_i) and !@user_custom_category_ids.include?(c_id.to_i) then
                pass = false
                break
              end
            end
            raise Exception unless pass
          end
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            c_unmatch = SidfmdRuleCMatchCategory.new(
                                                 sidfmd_rule_cond_id: rule_cond.id,
                                                 inv: true
                                                )
            if c_unmatch.save then
              cond_category_unmatch.each do |cat_id|
                SidfmdRuleCmCategorySub.create(
                                        sidfmd_rule_c_match_category_id: c_unmatch.id,
                                        sidfmd_category_id: cat_id.to_i
                                       )
              end
            end
          end
        end
        if cond_host_match then
          selected_hosts = SidfmdHost.where(:hostname => cond_host_match)
          if target_rule.sidfmd_category_id then #validation
            selected_hosts = selected_hosts.where(
              :id => SidfmdCategoriesHost.joins(:sidfmd_category)
                                         .where(:sidfmd_categories => { :is_system => false, :id => target_rule.sidfmd_category_id })
                                         .select(:sidfmd_host_id)
            )
          else
            selected_hosts = selected_hosts.where(
              :id => SidfmdCategoriesHost.joins(:sidfmd_category)
                                         .where(:sidfmd_categories => { :is_system => false })
                                         .select(:sidfmd_host_id)
            )
          end
          if selected_hosts.empty? or selected_hosts.size != cond_host_match.size then
            raise Exception
          end
          if !@sidfm_roles.include?(:admin) then #validation
            user_host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => @user_custom_category_ids).pluck(:sidfmd_host_id)
            if (selected_hosts.pluck(:id) - user_host_ids).any? then
              raise Exception
            end
          end
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            h_match = SidfmdRuleCMatchHost.new(
                                           sidfmd_rule_cond_id: rule_cond.id,
                                           inv: false
                                           )
            if h_match.save then
              selected_hosts.each do |host|
                SidfmdRuleCmHostSub.create(
                                    sidfmd_rule_c_match_host_id: h_match.id,
                                    sidfmd_host_id: host.id
                                   )
              end
            end
          end
        end
        if cond_host_unmatch then 
          selected_hosts = SidfmdHost.where(:hostname => cond_host_unmatch)
          if target_rule.sidfmd_category_id then #validation
            selected_hosts = selected_hosts.where(
              :id => SidfmdCategoriesHost.joins(:sidfmd_category)
                                         .where(:sidfmd_categories => { :is_system => false, :id => target_rule.sidfmd_category_id })
                                         .select(:sidfmd_host_id)
            )
          else
            selected_hosts = selected_hosts.where(
              :id => SidfmdCategoriesHost.joins(:sidfmd_category)
                                         .where(:sidfmd_categories => { :is_system => false })
                                         .select(:sidfmd_host_id)
            )
          end
          if selected_hosts.empty? or selected_hosts.size != cond_host_unmatch.size then
            raise Exception
          end
          if !@sidfm_roles.include?(:admin) then #validation
            user_host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => @user_custom_category_ids).pluck(:sidfmd_host_id)
            if (selected_hosts.pluck(:id) - user_host_ids).any? then
              raise Exception
            end
          end
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            h_unmatch = SidfmdRuleCMatchHost.new(
                                             sidfmd_rule_cond_id: rule_cond.id,
                                             inv: true
                                             )
            if h_unmatch.save then
              selected_hosts.each do |host|
                SidfmdRuleCmHostSub.create(
                                    sidfmd_rule_c_match_host_id: h_unmatch.id,
                                    sidfmd_host_id: host.id
                                    )
              end
            end
          end
        end
        if cond_host_syncing then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCHostSyncing.create(
              sidfmd_rule_cond_id: rule_cond.id,
              inv: cond_host_syncing == 'false'
            )
          end
        end
        if cond_status_match then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCStatus.create(
                              sidfmd_rule_cond_id: rule_cond.id,
                              status_id1: cond_status_match[0].to_i,
                              status_id2: cond_status_match.length == 2 ? cond_status_match[1] : nil,
                              )
          end
        end
        if cond_product_match then
          regist_cond_product_match_data(
            rule_id,
            :product_ids => cond_product_match["product-ids"],
            :version_ids => cond_product_match["version-ids"],
            :lproduct_ids => cond_product_match["lproduct-ids"],
            :lversion_ids => cond_product_match["lversion-ids"],
            :is_match => true
          )
        end
        if cond_product_unmatch then
          regist_cond_product_match_data(
            rule_id,
            :product_ids => cond_product_unmatch["product-ids"],
            :version_ids => cond_product_unmatch["version-ids"],
            :lproduct_ids => cond_product_unmatch["lproduct-ids"],
            :lversion_ids => cond_product_unmatch["lversion-ids"],
            :is_match => false
          )
        end
        if cond_cvss_exploitable then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCCvssExploitable.create(
                                       sidfmd_rule_cond_id: rule_cond.id,
                                       value: cond_cvss_exploitable["value"] == "true" ? true : false,
                                       is_change: cond_cvss_exploitable["is_change"] == "true" ? true : false
                                       )
          end
        end

        if cond_ctags then # カスタムタグの登録
          cond_ctags.each do |cond_ctag|
            enable_ids = cond_ctag['enable'].map{|ctag| ctag['id'].to_i }.uniq
            not_enable_ids = cond_ctag['not_enable'].map{|ctag| ctag['id'].to_i }.uniq
            role_admin = @sidfm_roles.include?(:admin)
            unless role_admin then #validation
              user_ctag_ids = SidfmdCtag.user_relatable_ctag_ids(role_admin, @user_custom_category_ids)
              if (enable_ids - user_ctag_ids).any? or (not_enable_ids - user_ctag_ids).any? then
                raise Exception
              end
            end
            if target_rule.sidfmd_category_id then #ルールのカテゴリが全体でない場合のvalidation
              enable_ids = SidfmdCtag.category_ctags(target_rule.sidfmd_category_id, enable_ids).pluck(:id)
              not_enable_ids = SidfmdCtag.category_ctags(target_rule.sidfmd_category_id, not_enable_ids).pluck(:id)
            else #ルールのカテゴリが全体の時
              enable_ids = SidfmdCtag.where(:id => enable_ids).pluck(:id)
              not_enable_ids = SidfmdCtag.where(:id => not_enable_ids).pluck(:id)
            end
            rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
            if rule_cond.save then
              rule_c_tag = SidfmdRuleCCtag.new(sidfmd_rule_cond_id: rule_cond.id)
              if enable_ids.any? or not_enable_ids.any? then
                rule_c_tag.sidfmd_rule_cm_ctags.build(
                  (
                    enable_ids.map{|ctag_id| { sidfmd_ctag_id: ctag_id, is_enable: true } } +
                    not_enable_ids.map{|ctag_id| { sidfmd_ctag_id: ctag_id, is_enable: false } }
                  )
                )
              end
              rule_c_tag.save
            end
          end
        end

        if cond_timeranges then
          cond_timeranges.each do |cond_timerange|
            if !([ 'hour', 'day', 'week', 'month' ].include?(cond_timerange['unit'])) then
              raise Exception
            end
            unless SidfmdRuleCTimerange::TARGET_VALUES.include?(cond_timerange['target'].to_i) then
              raise Exception
            end
            rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
            if rule_cond.save then
              SidfmdRuleCTimerange.create(
                sidfmd_rule_cond_id: rule_cond.id,
                target: cond_timerange['target'].to_i,
                value: cond_timerange['value'].to_i,
                unit: cond_timerange['unit'] ,
                inv: cond_timerange['except'].to_s == 'true' 
              )
            end
          end
        end
        #操作作成
        current_user_id = User.current.id
        action_send_mail_category = operation["send-mail-category"]
        action_send_mail_user = operation["send-mail-user"]
        action_status_change = operation["status-change"]
        action_due_date_change = operation["due-date-change"]
        action_send_syslog = operation["send-syslog"]
        action_set_ctag = operation["set-ctag"]

        if action_send_mail_category then
          sender = action_send_mail_category["sender"]
          action_target = action_send_mail_category["target"]
          action_flg = action_send_mail_category["proc_empty"]
          role_admin = @sidfm_roles.include?(:admin)
          if sender["userid"] then
            if get_filtered_mail_user_ids(_rule_category, sender["userid"]).size == 0 then
              raise Exception #不正またはユーザが削除されている
            end
          end
          action_send_mail_category["sendby"]["ids"].each do |id|
            if !role_admin && !@user_custom_category_ids.include?(id.to_i) then
              raise Exception #不正
            end
            rule_action = SidfmdRuleAction.new(
              sidfmd_rule_id: rule_id,
              target: action_target.to_i,
              proc_empty: action_flg == "true" ? true : false
            )
            if rule_action.save then
              SidfmdRuleAMailCategory.create(
                sidfmd_rule_action_id: rule_action.id,
                sidfmd_category_id: id == 'all' ? nil : id.to_i,
                from_user_id: sender["userid"] ? sender["userid"].to_i : nil,
                from_email: sender["address"] ? sender["address"] : nil,
                subject: action_send_mail_category["title"],
                body: action_send_mail_category["content"]
              )
            end
          end
        end
        if action_send_mail_user then
          sender = action_send_mail_user["sender"]
          action_target = action_send_mail_user["target"]
          action_flg = action_send_mail_user["proc_empty"]
          if sender["userid"] then
            if get_filtered_mail_user_ids(_rule_category, sender["userid"]).size == 0 then
              raise Exception #不正またはユーザが削除されている
            end
          end
          if action_send_mail_user["sendby"]["ids"] then
            action_send_mail_user["sendby"]["ids"] = get_filtered_mail_user_ids(_rule_category, action_send_mail_user["sendby"]["ids"])
            raise Exception if action_send_mail_user["sendby"]["ids"].size == 0 #不正またはユーザが削除されている
            action_send_mail_user["sendby"]["ids"].each do |id|
              rule_action = SidfmdRuleAction.new(
                                             sidfmd_rule_id: rule_id,
                                             target: action_target.to_i,
                                             proc_empty: action_flg == "true" ? true : false
                                            )
              if rule_action.save then
                SidfmdRuleAMailUser.create(
                                    sidfmd_rule_action_id: rule_action.id,
                                    user_id: id.to_i,
                                    from_user_id: sender["userid"] ? sender["userid"].to_i : nil,
                                    from_email: sender["address"] ? sender["address"] : nil,
                                    subject: action_send_mail_user["title"],
                                    body: action_send_mail_user["content"]
                                   )
              end
            end
          else
            action_send_mail_user["sendby"]["address"].each do |address|
              rule_action = SidfmdRuleAction.new(
                                             sidfmd_rule_id: rule_id,
                                             target: action_target.to_i,
                                             proc_empty: action_flg == "true" ? true : false
                                            )
              if rule_action.save then
                SidfmdRuleAMailUser.create(
                                    sidfmd_rule_action_id: rule_action.id,
                                    email: address,
                                    from_user_id: sender["userid"] ? sender["userid"].to_i : nil,
                                    from_email: sender["address"] ? sender["address"] : nil,
                                    subject: action_send_mail_user["title"],
                                    body: action_send_mail_user["content"]
                                   )
              end
            end
          end
        end
        if action_status_change then
          action_target = action_status_change["target"]
          action_flg = action_status_change["proc_empty"]
          rule_action = SidfmdRuleAction.new(
                                             sidfmd_rule_id: rule_id,
                                             target: action_target.to_i,
                                             proc_empty: action_flg == "true" ? true : false
                                            )
          if rule_action.save then
            SidfmdRuleAStatusChange.create(
                                    sidfmd_rule_action_id: rule_action.id,
                                    prev_status1: action_status_change["status_ids_from"][0].to_i,
                                    prev_status2: action_status_change["status_ids_from"][1] ? action_status_change["status_ids_from"][1].to_i : nil,
                                    status_id: action_status_change["status_ids_to"][0].to_i
                                   )
          end
        end
        if action_due_date_change then
          action_target = action_due_date_change["target"]
          action_flg = action_due_date_change["proc_empty"]
          rule_action = SidfmdRuleAction.new(
                                             sidfmd_rule_id: rule_id,
                                             target: action_target.to_i,
                                             proc_empty: action_flg == "true" ? true : false
                                            )
          if rule_action.save then
            SidfmdRuleADuedateChange.create(
                                     sidfmd_rule_action_id: rule_action.id,
                                     permit_update: action_due_date_change["permit-update"] == "true" ? true : false,
                                     fixed_date: action_due_date_change["date"] ? action_due_date_change["date"].to_date : nil,
                                     refdt_days: action_due_date_change["days"] ? action_due_date_change["days"].to_i : nil
                                     )
          end
        end
        if action_send_syslog then
          action_target = action_send_syslog["target"]
          action_flg = action_send_syslog["proc_empty"]
          protocol = action_send_syslog["protocol"]
          port = action_send_syslog["port_number"]
          hostname = action_send_syslog["hostname"]
          facility = action_send_syslog["facility"]
          severity = action_send_syslog["severity"]
          address = action_send_syslog["address"]
          message = action_send_syslog["message"]
          selected_address = action_send_syslog["selected_address"] 
          if !['udp', 'tcp'].include?(protocol) || !SidfmdRule::SYSLOG_FACILITIES.include?(facility) ||
              /\A[0-9a-zA-Z._-]+\z/ !~ hostname || !SidfmdRule::SYSLOG_SEVERITIES.include?(severity) ||
             (selected_address != 'hostname' && selected_address != 'ip_address') || /\A[0-9]+\z/ !~ port
          then
            raise Exception
          end
          port = port.to_i
          if port < 0 || port > 65535 then
            raise Exception
          end
          if (selected_address == 'hostname' && /\A[0-9a-zA-Z._-]+\z/ !~ address) || 
             (selected_address == 'ip_address' && /\A((25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.){3}(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\z/ !~ address) 
          then
            raise Exception
          end
          is_message_pass = SidfmdRuleASendSyslog.validate_keywords(message)
          raise Exception unless is_message_pass
          rule_action = SidfmdRuleAction.new(
                                             sidfmd_rule_id: rule_id,
                                             target: action_target.to_i,
                                             proc_empty: action_flg == "true" ? true : false
                                            )
          if rule_action.save then
            SidfmdRuleASendSyslog.create(
                                    sidfmd_rule_action_id: rule_action.id,
                                    protocol: protocol,
                                    target: (selected_address == 'ip_address' ? SidfmdRuleASendSyslog::IP_ADDRESS_IDX + address : address),#編集時にホスト名かipアドレスかで表示する要素がことなるため、ipアドレスの場合は%ip_address%を 接頭語にいれて保存する。%はホスト名に入れられない文字でチェックしてあるため、%ip_address%の文字列はここでしか保存できないようになっています。
                                    port: port,
                                    hostname: hostname,
                                    ident: action_send_syslog["ident"],
                                    facility: facility,
                                    severity: severity,
                                    message: message
                                  )  
          end   
        end

        if action_set_ctag then #カスタムタグ設定
          enable_ids = action_set_ctag['enable'].map{|ctag| ctag['id'].to_i }.uniq
          not_enable_ids = action_set_ctag['not_enable'].map{|ctag| ctag['id'].to_i }.uniq
          if !enable_ids.any? and !not_enable_ids.any? then
            raise Exception #invalid
          end
          role_admin = @sidfm_roles.include?(:admin)
          unless role_admin then #validation
            user_ctag_ids = SidfmdCtag.user_relatable_ctag_ids(role_admin, @user_custom_category_ids)
            if (enable_ids - user_ctag_ids).any? or (not_enable_ids - user_ctag_ids).any? then
              raise Exception
            end
          end
          if target_rule.sidfmd_category_id then #ルールのカテゴリが全体でない場合のvalidation
            enable_ids = SidfmdCtag.category_ctags(target_rule.sidfmd_category_id, enable_ids).pluck(:id)
            not_enable_ids = SidfmdCtag.category_ctags(target_rule.sidfmd_category_id, not_enable_ids).pluck(:id)
          else #ルールのカテゴリが全体の時
            enable_ids = SidfmdCtag.where(:id => enable_ids).pluck(:id)
            not_enable_ids = SidfmdCtag.where(:id => not_enable_ids).pluck(:id)
          end
          if !enable_ids.any? and !not_enable_ids.any? then
            raise Exception 
          end
          rule_action = SidfmdRuleAction.new(
            sidfmd_rule_id: rule_id,
            target: action_set_ctag['target'].to_i,
            proc_empty: action_set_ctag['proc_empty'] == "true"
          )
          if rule_action.save then
            rule_action_c_tag = SidfmdRuleACtag.new(sidfmd_rule_action_id: rule_action.id)
            rule_action_c_tag.sidfmd_rule_am_ctags.build(
              (
                enable_ids.map{|ctag_id| { sidfmd_ctag_id: ctag_id, is_enable: true } } +
                not_enable_ids.map{|ctag_id| { sidfmd_ctag_id: ctag_id, is_enable: false } }
              )
            )
            rule_action_c_tag.save
          end
        end

      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'ルールの登録に失敗しました。' } and return
    end

    #変更の場合に、ルールのログを復元して設定変更のログを追加する
    begin
      if _log_rule_id then
        SidfmdRule.transaction do
          rule = SidfmdRule.lock.find(_log_rule_id) 
          create_updated_rule_logs(
            rule,
            :prev_trigger => _prev_trigger,
            :prev_conds => _prev_conds,
            :prev_actions => _prev_actions,
            :is_updated_status => _is_updated_rule_status,
            :is_updated_settings => true
          )
        end
      end
    rescue Exception => e
      log_exception(e)
    end
    return true
  end

  def get_users_custom_categories
    if @sidfm_roles.include?(:admin) then
      c_categories = @all_sidfm_categories.map{|c| [c.id, c.name]}
    else
      c_categories = User.current
                         .categories
                         .map{|c| [c.id, c.name]}
    end
    return c_categories.sort{|a, b| a[1] <=> b[1]}.to_h
  end

  #ソート
  def sort_rules
    last_index = @rules[:order].rindex('_')
    col = @rules[:order].slice(0..last_index - 1)
    order = @rules[:order].slice(last_index + 1..@rules[:order].size - 1)
=begin
    case col
    when "rule-name" then
      col = "name"
      @all_rules = @all_rules.order('LOWER(name COLLATE "C")' + (order == 'asc' ? ' ASC' : ' DESC'))
    when "rule-category" then
      case order
      when "asc"
        @all_rules = @all_rules.joins('LEFT OUTER JOIN sidfmd_categories ON sidfmd_rules.sidfmd_category_id = sidfmd_categories.id')
                               .order('LOWER(sidfmd_categories.name COLLATE "C") NULLS FIRST') #「全体」(null)を優先する
                               .order('LOWER(sidfmd_rules.name COLLATE "C")')
      when "desc"
        @all_rules = @all_rules.joins('LEFT OUTER JOIN sidfmd_categories ON sidfmd_rules.sidfmd_category_id = sidfmd_categories.id')
                               .order('LOWER(sidfmd_categories.name COLLATE "C") DESC NULLS LAST')
                               .order('LOWER(sidfmd_rules.name COLLATE "C") DESC')
      end
    when "rule-active" then
      col = "flg_enable"
      case order
      when "asc"
        @all_rules = @all_rules.order(col => :asc)
                               .order('LOWER(sidfmd_rules.name COLLATE "C")')
      when "desc"
        @all_rules = @all_rules.order(col => :desc)
                               .order('LOWER(sidfmd_rules.name COLLATE "C") DESC')
      end
    end
=end
    #COLATTE "C"を使うとfor SELECT DISTINCT, ORDER BY expressions must appear in select listエラーになるため、rubyで処理する
    case col
    when "rule_name" then
      @all_rules = @all_rules.sort_by{|rule| rule.name.downcase}
    when "rule_category" then
      categories_order = {}
      SidfmdCategory.where(:is_system => false).order('LOWER(name COLLATE "C")').each_with_index do |cat, idx|
        categories_order[cat.id] = idx
      end
      @all_rules = @all_rules.sort{|a_rule, b_rule| 
        a = (categories_order[a_rule.sidfmd_category_id] ? categories_order[a_rule.sidfmd_category_id] : -1)
        b = (categories_order[b_rule.sidfmd_category_id] ? categories_order[b_rule.sidfmd_category_id] : -1)
        r = a <=> b
        if r == 0 then
          r = a_rule.name.downcase <=> b_rule.name.downcase
        end
        r
      }
    when "rule_position"
      @all_rules = @all_rules.sort{|b, a| 
        r = a.position.to_i <=> b.position.to_i
      }
    when "rule_active" then
      @all_rules = @all_rules.sort{|a_rule, b_rule| 
        a = (a_rule.flg_enable ? 0 : 1)
        b = (b_rule.flg_enable ? 0 : 1)
        r = a <=> b
        if r == 0 then
          r = a_rule.name.downcase <=> b_rule.name.downcase
        end
        r
      }
    end
    @all_rules.reverse! if order == "desc"
  end

  #検索処理
  def search_items
    rule_name = @rules[:filters][:search_rulename]
    rule_category = @rules[:filters][:search_category]
    rule_trigger = @rules[:filters][:search_trigger]
    rule_cond = @rules[:filters][:search_cond]
    rule_action = @rules[:filters][:search_ope]
    rule_flg  = @rules[:filters][:search_flg]

    if rule_name && rule_name.length > 0 then
      @all_rules = @all_rules.where('name ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, rule_name)}%")
    end

    if rule_category && rule_category.length > 0 && rule_category.to_i != 0 then
      if rule_category == "-1" then
        @all_rules = @all_rules.where(:sidfmd_category_id => nil)
      else
        @all_rules = @all_rules.where(:sidfmd_category_id => rule_category)
      end
    end

    if rule_trigger && rule_trigger.length > 0 then
      case rule_trigger 
        when "periodic_hour" then trigger = SidfmdRuleTpHour
        when "periodic_interval" then trigger = SidfmdRuleTpInterval
        when "vulsync" then trigger = SidfmdRuleTVulsync 
      end
      @all_rules = @all_rules.joins(:sidfmd_rule_trigger).where("sidfmd_rule_triggers.id IN (?)", trigger.all.map{|t| t.sidfmd_rule_trigger_id})
    end
    if rule_cond && rule_cond.length > 0 then
      case rule_cond 
        when "issue-update" then cond = SidfmdRuleCIssueUpdate.all
        when "cvss-exploitable" then cond = SidfmdRuleCCvssExploitable.all
        when "sri-above" then cond = SidfmdRuleCSriAbove.all
        when "sri-below" then cond = SidfmdRuleCSriBelow.all
        when "sri-warning" then cond = SidfmdRuleCWarning.where(:is_cvss => false)
        when "cvss-above" then cond = SidfmdRuleCCvssAbove.all
        when "cvss-below" then cond = SidfmdRuleCSriBelow.all
        when "cvss-warning" then cond = SidfmdRuleCWarning.where(:is_cvss => true)
        when "cvss-notreg" then cond = SidfmdRuleCCvssNotreg.all
        when "cvss-vector" then cond = SidfmdRuleCCvssVector.all
        when "status-match" then cond = SidfmdRuleCStatus.all
        when "product-match" then cond = SidfmdRuleCMatchApp.where(:inv => false)
        when "product-unmatch" then cond = SidfmdRuleCMatchApp.where(:inv => true) 
        when "due-date-expired" then cond = SidfmdRuleCDuedateExpired.all
        when "due-date-approach" then cond = SidfmdRuleCDuedateApproaching.all
        when "due-date-undef" then cond = SidfmdRuleCDuedateUndef.all
        when "history-create" then cond = SidfmdRuleCHistoryPost.all
        when "category-match" then cond = SidfmdRuleCMatchCategory.where(:inv => false)
        when "category-unmatch" then cond = SidfmdRuleCMatchCategory.where(:inv => true)
        when "host-match" then cond = SidfmdRuleCMatchHost.where(:inv => false)
        when "host-unmatch" then cond = SidfmdRuleCMatchHost.where(:inv => true)
        when "host-syncing" then cond = SidfmdRuleCHostSyncing.all
        when "timerange" then cond = SidfmdRuleCTimerange.all
        when "ctag" then cond = SidfmdRuleCCtag.all
      end
      @all_rules = @all_rules.joins(:sidfmd_rule_conds).where("sidfmd_rule_conds.id IN (?)", cond.select(:sidfmd_rule_cond_id)).uniq
    end

    if rule_action && rule_action.length > 0 then
      case rule_action
        when "send-mail-user" then action = SidfmdRuleAMailUser
        when "send-mail-category" then action = SidfmdRuleAMailCategory
        when "status-change" then action = SidfmdRuleAStatusChange
        when "due-date-change" then action = SidfmdRuleADuedateChange
        when "send-syslog" then action = SidfmdRuleASendSyslog
        when "set-ctag" then action = SidfmdRuleACtag
      end
      @all_rules = @all_rules.joins(:sidfmd_rule_actions).where("sidfmd_rule_actions.id IN (?)", action.all.map{|a| a.sidfmd_rule_action_id}).uniq
    end
    if rule_flg && rule_flg.length > 0 then
      if rule_flg == "enabled" then
        @all_rules = @all_rules.where(:flg_enable => true)
      else 
        @all_rules = @all_rules.where(:flg_enable => false)
      end
    end
  end

  #データ作成
  def create_rule_info(rule)#create_content
    #rules = []
    #@all_rules.each do |rule|
      rule_name = rule.name
      rule_category = rule.sidfmd_category_id
      is_enable = rule.flg_enable
      trigger = rule.sidfmd_rule_trigger.detail
      trigger_name = trigger.class.table_name
      conditions = rule.sidfmd_rule_conds
      actions = rule.sidfmd_rule_actions

      rule_info  = {
        :rule_id => rule.id,
        :name => rule_name,
        :position => rule.position,
        :category_name => rule_category.nil? ? '全体' : @all_sidfm_categories.find(rule_category.to_i).name,
        :trigger => if trigger_name == "sidfmd_rule_tp_hours" then
                      ["毎日実行-時間指定", "実行時間: #{trigger.hour.to_s}時"]
                    elsif trigger_name == "sidfmd_rule_tp_intervals" then
                       unit = trigger.unit
                       case unit
                       when "hour" then
                         unit = "時間毎"
                       when "day" then
                         unit = "日毎"
                       when "week" then
                         unit = "週毎"
                       when "month" then
                         unit = "ヵ月毎"
                       end
                       ["定期実行-間隔指定", "実行間隔: #{trigger.value.to_s + unit}"]
                    else
                      category_id = trigger.sidfmd_category_id
                      host_id = trigger.sidfmd_host_id
                      trigger_text = if !category_id && !host_id then
                                       "対象カテゴリ: 全体"
                                     elsif host_id then
                                       t_host = SidfmdHost.find(host_id)
                                       "対象ホスト: #{t_host.ext_name ? t_host.ext_name : t_host.hostname}"
                                     else 
                                       "対象カテゴリ: #{@all_sidfm_categories.find(category_id).name}"
                                     end
                      ["脆弱性同期時 実行", trigger_text]
                    end,
        :cond_num => conditions.length,
        :ope_num => actions.map{|a| a.detail.class.table_name}.uniq.size,
        :logs => rule.logs,
        :enabled => is_enable,
        :param => create_rule_param(rule_name, rule_category, rule.position, is_enable, trigger, conditions, actions, rule.updated_at)
      }
      #rules << rule_info
    #end
    #return rules
  end

  def get_preset_rules_data
    status_not_conf = SidfmdIssue::STATUS_NOTCONFIRMED
    status_on_going = SidfmdIssue::STATUS_ONGOING
    status_not_affect = SidfmdIssue::STATUS_NOTAFFECT
    sri_high = SidfmdIssueSri::VALUE_HIGH
    sri_low = SidfmdIssueSri::VALUE_LOW
    target_change = SidfmdRuleAction::TARGET_CHANGE
    target_vuls = SidfmdRuleAction::TARGET_VULS
    preset_mail_sender = Setting.mail_from
    user_category_ids = @c_categories.keys
    #報告者権限の場合は、ユーザのカテゴリのカテゴリ名でsortした初めのカテゴリをセット
    rule_category_id = if @sidfm_roles.include?(:admin) then
                         'all'
                       else
                         (user_category_ids.size > 0 ? user_category_ids[0] : '')
                       end
    rule_params = [
      {
        "rule_name" => "SRI値が「高」以上の脆弱性をメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger" =>{"hostname"=>"", "period"=>"vulsync","value"=>"vulsync", "unit"=>"", "category"=> rule_category_id},
        "condition" =>{
            "sri-above" => sri_high,
            "status-match" => [status_not_conf, status_on_going]
        },
        "action" =>{
          "send-mail-category" =>{
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "SRI値が「高」以上の脆弱性",
            "content" => "新たに検出された脆弱性は以下の通りです。\n\n登録された脆弱性\n%VULSYNC_REGI_MATCH:T3%\n他、%VULSYNC_REGI_MATCH:C-3%件\n\n更新された脆弱性\n%VULSYNC_RENEW_MATCH:T3%\n他、%VULSYNC_RENEW_MATCH:C-3%件\n\n削除された脆弱性\n%VULSYNC_DEL_MATCH:T3%\n他、%VULSYNC_DEL_MATCH:C-3%件",
            "sender" => {"address" => preset_mail_sender},
            "target" => target_change.to_s,
            "proc_empty" => "false"
          }
        },
        "description" => "脆弱性同期時に、SRI値が「高」または「重大」の脆弱性が記載されたメールを送信します。"
      },
      {
        "rule_name" => "CVSS値が 7 以上の脆弱性をメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger" => {"hostname"=>"", "period"=>"vulsync","value"=>"vulsync", "unit"=>"", "category"=> rule_category_id},
        "condition" => {
          "cvss-above" => {"val"=>7.0, "include_notreg"=>"false", "flg_exploitable"=>"all"},
          "status-match" => [status_not_conf, status_on_going]
        },
        "action" =>{
          "send-mail-category" =>{
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "CVSS値が7以上の脆弱性",
            "content" => "新たに検出された脆弱性は以下の通りです。\n\n登録された脆弱性\n%VULSYNC_REGI_MATCH:T3%\n他、%VULSYNC_REGI_MATCH:C-3%件\n\n更新された脆弱性\n%VULSYNC_RENEW_MATCH:T3%\n他、%VULSYNC_RENEW_MATCH:C-3%件\n\n削除された脆弱性\n%VULSYNC_DEL_MATCH:T3%\n他、%VULSYNC_DEL_MATCH:C-3%件",
            "sender" => {"address" => preset_mail_sender},
            "target" => target_change.to_s,
            "proc_empty" => "false"
          }
        },
        "description" => "脆弱性同期時に、CVSS値が 7.0 以上の脆弱性が記載されたメールを送信します。"
      },
      {
        "rule_name" => "7 日毎に未対策の脆弱性チケットをメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger"=>{"hostname"=>"", "period"=>"periodic-interval", "value"=>7, "unit"=>"day", "category"=>""},
        "condition"=>{"status-match"=>[status_not_conf]},
        "action"=>{
          "send-mail-category"=>{
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "未対策の脆弱性",
            "content" => "未対策の脆弱性は以下の通りです。\n\n%VUL_LINKS_MATCH:T3%\n\nその他、%VUL_LINKS_MATCH:C-3%件検出されています。",
            "sender" => {"address" => preset_mail_sender},
            "target" => target_vuls.to_s,
            "proc_empty" => "true"
          }
        },
        "description" => "7 日毎に、ステータスが「未対策」となっている脆弱性が記載されたメールを送信します。"
      },
      {
        "rule_name" => "7 日毎に更新された脆弱性チケットをメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger" => {"hostname"=>"", "period"=>"periodic-interval", "value"=>7, "unit"=>"day", "category"=>""},
        "condition" => {"history-create" => {"val" => 1}},
        "action" => {
          "send-mail-category" => {
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "更新された脆弱性チケット",
            "content" => "更新された脆弱性チケットは以下の通りです。\n\n%VUL_LINKS_MATCH:T3%\n\nその他、%VUL_LINKS_MATCH:C-3%件検出されています。",
            "sender" => {"address" => preset_mail_sender},
            "target" => target_vuls.to_s,
            "proc_empty" => "true"
          }
        },
        "description" => "7 日毎に、脆弱性チケットの更新が１件以上ある脆弱性を記載したメールを送信します。"
      },
      {
        "rule_name" => "対策予定日を過ぎた未対策の脆弱性チケットをメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger" => {"hostname"=>"", "period"=>"periodic-hour", "value"=>3, "unit"=>"", "category"=>""},
        "condition" => {"due-date-expired" => "due-date-expired"},
        "action" => {
          "send-mail-category" => {
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "対策予定日を過ぎた未対策の脆弱性チケット",
            "content" => "対策予定日を過ぎた未対策の脆弱性チケットは以下の通りです。\n\n%VUL_LINKS_MATCH:T3%\n\nその他、%VUL_LINKS_MATCH:C-3%件検出されています。",
            "sender" => {"address" => preset_mail_sender},
            "target" => target_vuls.to_s,
            "proc_empty" => "true"
          }
        },
        "description" => "毎日午前 3 時に、対策予定日を過ぎた未対策の脆弱性チケットが記載されたメールを送信します。"
      },
      {
        "rule_name" => "対策予定日が 5 日以内の脆弱性チケットをメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger" => {"hostname"=>"", "period"=>"periodic-hour", "value"=>3, "unit"=>"", "category"=>""},
        "condition" => {"due-date-approach"=>{"val"=>5}},
        "action" => {
          "send-mail-category" => {
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "対策予定日が 5 日以内の脆弱性チケット",
            "content" => "対策予定日が 5 日以内の脆弱性チケットは以下の通りです。\n\n%VUL_LINKS_MATCH:T3%\n\nその他、%VUL_LINKS_MATCH:C-3%件検出されています。",
            "sender" => {"address"=> preset_mail_sender},
            "target" => target_vuls.to_s,
            "proc_empty" => "true"
          }
        },
        "description" => "毎日午前 3 時に、対策予定日が 5 日以内に設定されている脆弱性チケットが記載されたメールを送信します。"
      },
     {
       "rule_name" => "SRI値が「低」の脆弱性チケットを「影響無」にする", 
       "is_enable" => true, 
       "category" => rule_category_id,
       "trigger" => {"hostname"=>"", "period"=>"vulsync", "value"=>"vulsync", "unit"=>"", "category"=> rule_category_id},
       "condition" => {
         "sri-below" => sri_low,
         "status-match" => [status_not_conf, status_on_going]
       }, 
       "action" => {
         "status-change" => {
           "status_ids_from" => [status_not_conf, status_on_going], 
           "status_ids_to" => [status_not_affect],
           "target" => target_change.to_s,
           "proc_empty" => "true"
         }
       },
       "description" => "脆弱性同期時に、SRI値が「低」となっている脆弱性チケットのステータスを「影響無」に変更します。"
     },
     {
       "rule_name" => "CVSS値が 3.9 以下の脆弱性チケットを「影響無」に変更する",
       "is_enable" => true,
       "category" => rule_category_id,
       "trigger" => {"hostname"=>"", "period"=>"vulsync", "value"=>"vulsync", "unit"=>"", "category"=> rule_category_id},
       "condition" => {
         "cvss-below" => {"val"=>3.9, "include_notreg"=>"false", "flg_exploitable"=>"all"},
         "status-match" => [status_not_conf, status_on_going]
       },
       "action" => {
         "status-change" => {
           "status_ids_from" => [status_not_conf, status_on_going], 
           "status_ids_to" => [status_not_affect],
           "target" => target_change.to_s,
           "proc_empty" => "true"
         }
       },
       "description" => "脆弱性同期時に、CVSS値が 3.9 以下の脆弱性チケットのステータスを「影響無」に変更します。"
     },
     {
       "rule_name" => "SRI値が「高」以上の脆弱性チケットに対策予定日として 20 日後を設定する",
       "is_enable"=> true,
       "category" => rule_category_id,
       "trigger"=> {"hostname"=>"", "period"=>"vulsync", "value"=>"vulsync", "unit"=>"", "category"=> rule_category_id},
       "condition" => {
         "sri-above" => sri_high,
         "status-match" => [status_not_conf, status_on_going]
       },
       "action" => {
         "due-date-change" => {
           "days" => 20, 
           "permit-update" => "false",
           "target" => target_change.to_s,
           "proc_empty" => "true"
         }
       },
       "description" => "脆弱性同期時に、SRI値が「高」または「重大」となっている脆弱性チケットの対策予定日を 20 日後に設定します。"
     },
     {
       "rule_name" => "CVSS値が 7.0 以上の脆弱性チケットに対策予定日として 20 日後を設定する",
       "is_enable" => true, 
       "category" => rule_category_id, 
       "trigger" => {"hostname"=>"", "period"=>"vulsync", "value"=>"vulsync", "unit"=>"", "category"=>rule_category_id}, 
       "condition" => {
         "cvss-above" => {"val"=>7.0, "include_notreg"=>"false", "flg_exploitable"=>"all"},
         "status-match" => [status_not_conf, status_on_going]
       }, 
       "action" => {
         "due-date-change"=>{
           "days"=>20,
           "permit-update"=>"false",
           "target" => target_change.to_s,
           "proc_empty" => "true"
         }
       },
       "description" => "脆弱性同期時に、CVSS値が 7.0 以上の脆弱性の対策予定日を 20 日後に設定します。"
     }
    ]
  end

  def get_mail_preset
    list_values = SidfmdRuleAction::TARGETS
    preset_list = [
     {
       :text => ["条件に適合したホストの一覧（最大10件）を送信する", "(データが存在しない場合、メールを送信しない)"],
       :target => list_values[1],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合するホストの一覧",
       :message => "ルール %RULE_NAME% の条件に適合するホストは以下の通りです。\n\n" +
                   "対象ホスト数: %HOST_MATCH:C% 件\n\n" +
                   "ホスト一覧(最大10件):\n" +
                   "%DISPLAY_NAME_MATCH:T10%",
     },
     {
       :text => ["条件に適合したホストの一覧（最大10件）を送信する","(データが存在しない場合でも、メールを送信する)"],
       :target => list_values[1],
       :proc_empty => true,
       :subject => "%RULE_NAME%の条件に適合するホストの一覧",
       :message => "ルール %RULE_NAME% の条件に適合するホストは以下の通りです。\n\n" +
                   "対象ホスト数: %HOST_MATCH:C% 件\n\n" +
                   "ホスト一覧(最大10件):\n" +
                   "%DISPLAY_NAME_MATCH:T10%"
     },
     {
       :text => ["条件に適合した脆弱性の一覧（最大10件）を送信する", "(データが存在しない場合、メールを送信しない)"],
       :target => list_values[0],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合する脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する脆弱性は以下の通りです。\n\n" +
                   "対象脆弱性数: %VULINFO_MATCH:C% 件\n\n" +
                   "脆弱性一覧(最大10件):\n" +
                   "%VULINFO_MATCH:T10%"
     },
     {
       :text => ["条件に適合した脆弱性の一覧（最大10件）を送信する", "(データが存在しない場合でも、メールを送信する)"],
       :target => list_values[0],
       :proc_empty => true,       
       :subject => "%RULE_NAME%の条件に適合する脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する脆弱性は以下の通りです。\n\n" +
                   "対象脆弱性数: %VULINFO_MATCH:C% 件\n\n" +
                   "脆弱性一覧(最大10件):\n" +
                   "%VULINFO_MATCH:T10%"
     },
     {
       :text => ["条件に適合した新規登録脆弱性を送信する", "(データが存在しない場合、メールを送信しない)"],
       :target => list_values[2],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合する新規登録脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する新規登録の脆弱性は以下の通りです。\n\n" +
                   "対象脆弱性数: %VULSYNC_REGI_MATCH:C% 件\n\n" +
                   "脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_REGI_MATCH:T100%",
       :trigger => "vulsync" 
     },
     {
       :text => ["条件に適合した更新脆弱性を送信する", "(データが存在しない場合、メールを送信しない)"],    
       :target => list_values[3],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合する更新脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する更新脆弱性は以下の通りです。\n\n" +
                   "対象脆弱性数: %VULSYNC_RENEW_MATCH:C% 件\n\n" +
                   "脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_RENEW_MATCH:T100%",
       :trigger => "vulsync"

     },
     {
       :text => ["条件に適合した削除脆弱性を送信する", "(データが存在しない場合、メールを送信しない)"],
       :target => list_values[4],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合する削除脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する削除脆弱性は以下の通りです。\n\n" +
                   "対象脆弱性数: %VULSYNC_DEL_MATCH:C% 件\n\n" +
                   "脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_DEL_MATCH:T100%",
       :trigger => "vulsync"
     },
     {
       :text => ["条件に適合した登録・更新・削除脆弱性を送信する", "(データが存在しない場合、メールを送信しない)"],
       :target => list_values[5],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合する登録・更新・削除脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する登録・更新・削除登録の脆弱性は以下の通りです。\n\n" +    
                   "対象登録脆弱性数: %VULSYNC_REGI_MATCH:C% 件\n" +
                   "対象更新脆弱性数: %VULSYNC_RENEW_MATCH:C% 件\n" +
                   "対象削除脆弱性数: %VULSYNC_DEL_MATCH:C% 件\n" +
                   "登録脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_REGI_MATCH:T100%\n\n" +
                   "更新脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_RENEW_MATCH:T100%\n\n" +
                   "削除脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_DEL_MATCH:T100%",
       :trigger => "vulsync"
     }
    ]

  end

  #条件のCVSS指標のデータ
  #パラメータやUIの表示、validationで使用する
  def get_cvss_vectors
    vectors = SidfmdIssueCve.get_cvss_v3_vectors
    vectors.each do |item, content|
      content['values']['NONE'] = '未設定'
    end
    vectors
  end

  FILTER_KEYS = {}

  #フィルターの初期化
  def init_filter_keys
    if FILTER_KEYS.size == 0 then
      FILTER_KEYS[:search_rulename] = ''
      FILTER_KEYS[:search_category] = '0'
      FILTER_KEYS[:search_trigger] = ''
      FILTER_KEYS[:search_cond] = ''
      FILTER_KEYS[:search_ope] = ''
      FILTER_KEYS[:search_flg] = ''
    end
  end

  def check_filters
    init_filter_keys
    FILTER_KEYS.each do |key, val|
      unless @rules[:filters].has_key?(key) then
        @rules[:filters][key] = val
      end
    end
    @default_filters = FILTER_KEYS
  end

  def parse_params
    filtered_params = params.fetch(:pages, {}).permit(:offset, :per_page, :order, :filters, :checked, :all_checked)
    @rules = {}
    @rules[:per_page] = (@summary_table_lines || filtered_params[:per_page] || 10).to_i
    @rules[:offset] = (filtered_params[:offset] || 0).to_i
    @rules[:filters] = JSON.parse(filtered_params[:filters] || '{}').symbolize_keys
    @rules[:checked] = JSON.parse(filtered_params[:checked] || '[]')
    @rules[:all_checked] = ((filtered_params[:all_checked] || '') == 'true')
    @rules[:order] = (filtered_params[:order] || 'rule_name_asc')
    @rules[:search_rulename] = @rules[:filters][:search_rulename] || ''
    @rules[:search_category] = @rules[:filters][:search_category] || '0'
    @rules[:search_trigger] = @rules[:filters][:search_trigger] || ''
    @rules[:search_cond] = @rules[:filters][:search_cond] || ''
    @rules[:search_ope] = @rules[:filters][:search_ope] || ''
    @rules[:search_flg] = @rules[:filters][:search_flg] || '' 
  end

  def create_summary_rule_html(rule, **opts)
    data = {
      :chkbox => view_context.create_summary_rule_chkbox(rule[:rule_id], opts[:user].admin, opts[:checked]),
      :rule_name => view_context.create_summary_rule_name(rule[:name]),
      :rule_category => view_context.create_summary_rule_category(rule[:category_name], (rule[:param] ? rule[:param][:category] : nil)),
      :trigger => view_context.create_summary_rule_trigger(rule[:trigger]),
      :condition => view_context.create_summary_rule_condition(rule),
      :operation => view_context.create_summary_rule_operation(rule),
      :rule_position => view_context.create_summary_rule_priority(rule[:position]),
      :log => view_context.create_summary_rule_log(rule[:name], rule[:logs]),
      :rule_active => view_context.create_summary_rule_status(rule[:enabled]),
      :rule_edit => view_context. create_summary_rule_edit(rule[:rule_id], rule[:name])
    }
  end
 
  public

  def index 
    parse_params
    check_filters
    @all_sidfm_categories = SidfmdCategory.where(:is_system => false)
    @c_categories = get_users_custom_categories()
    #@accessible_users = get_accessible_users()
    @all_triggers = SidfmdRule::RULE_TRIGGER_NAMES
    @all_conds = SidfmdRule::RULE_COND_NAMES
    @all_opes = SidfmdRule::RULE_ACTION_NAMES
    #最大登録件数
    @maximum_limit = Sidfm.max_rules
    #プリセット一覧
    @data_for_preset = get_preset_rules_data
    #メールのプリセット一覧
    @data_for_mail_preset = get_mail_preset
    #CVSS指標
    @cvss_vectors = get_cvss_vectors
    #システムカテゴリ
    @system_categories = {}
    #カテゴリの概要一覧
    @category_descs = {}
    user_cats = if @sidfm_roles.include?(:admin) then
                  SidfmdCategory
                else
                  SidfmdCategory.where("is_system IS TRUE OR id IN (?)", @c_categories.keys)
                end
    user_cats.order(:id).select(:id, :name, :desc, :is_system).each do |cat|
      if cat.is_system then
        @system_categories[cat.id] = cat.name
      end
      @category_descs[cat.id] = { :name => cat.name, :desc => cat.desc }
    end
    #ブラウザバックかどうか判断するための処理 チェックボックスの描画のバグを防ぐために使用
    #if !session['chkbok_ids'] then
    #  session['chkbok_ids'] = {}
    #end
    #if !session['chkbok_ids']['rule'] then
    #  session['chkbok_ids']['rule'] = {}
    #end
    #session['chkbok_ids']['rule']['is_browser_back'] = false
  end

  def create
    if SidfmdRule.count == Sidfm.max_rules then
      render :layout => false, :json => { :err_message => "ルールの登録は#{SidfmdRule::Sidfm.max_rules}件までとなっています" } and return
    end
    f_params = params.permit(:rulename, :category, :flg, :trigger, :condition, :operation, :rule_position)
    rulename = f_params[:rulename]
    category = f_params[:category]
    flg = f_params[:flg]
    rule_position = f_params[:rule_position]
    begin 
      trigger = JSON.parse(f_params[:trigger])
      condition = JSON.parse(f_params[:condition]) 
      operation = JSON.parse(f_params[:operation]) 
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => "入力が不正です。" } and return
    end
    if !rulename || !category || !flg || !trigger || !condition || !operation then
      render :layout => false, :json => { :err_message => "ルールの登録に失敗しました。" } and return
    end
    if category == "all" then
      category = nil
    else
      c_id = category.to_i
      if @user_custom_category_ids.include?(c_id) then
        category = c_id
      else
        render :layout => false, :json => { :err_message => "入力が不正です。" } and return
      end
    end
    if SidfmdRule.where(:name => rulename).length > 0 then
      render :layout => false, :json => { :err_message => "既に同名のルールが存在します" } and return
    end
    create_rule = set_rule(rulename, category, flg, rule_position, trigger, condition, operation, 'create', nil)
    if create_rule == true then
      Rails.cache.clear
      render :layout => false, :json => { :err_message => '' }
    else
      #エラーになる場合は、def set_rule内でrenderしている
      #render :layout => false, :json => { :err_message => "ルールの登録に失敗しました。" }
    end
  end

  def modify
    f_params = params.permit(:rule_id, :rulename, :category, :flg, :trigger, :condition, :operation, :updated_at, :only_flg_changed, :rule_position)
    rule_id = f_params[:rule_id]
    rulename = f_params[:rulename]
    category = f_params[:category]
    flg = f_params[:flg]
    rule_position = f_params[:rule_position]
    begin
      trigger = JSON.parse(f_params[:trigger])
      condition = JSON.parse(f_params[:condition])
      operation = JSON.parse(f_params[:operation])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => "入力が不正です。" } and return
    end
    if !rule_id || !rulename || !category || !flg || !trigger || !condition || !operation then
      render :layout => false, :json => { :err_message => "ルールの編集に失敗しました。" } and return
    end
    if category == "all" then
      category = nil
    else
      c_id = category.to_i
      if @user_custom_category_ids.include?(c_id) then
        category = c_id
      else
        render :layout => false, :json => { :err_message => "入力が不正です。" } and return
      end
    end
    updated_at = f_params[:updated_at]
    if !updated_at || updated_at.empty? then
      render :layout => false, :json => { :err_message => "入力が不正です。" } and return
    end
    begin 
      updated_at = updated_at.in_time_zone + 1
    rescue Exception => e
      render :layout => false, :json => { :err_message => "入力が不正です。" } and return
    end 
    edit_rule = set_rule(rulename, category, flg, rule_position, trigger, condition, operation, 'modify', rule_id, updated_at, f_params[:only_flg_changed])
    if edit_rule then
      Rails.cache.clear
      render :layout => false, :json => { :err_message => '' }
    end
  end

  def delete
    begin 
      f_params = params.permit(:delete_id)
      delete_id = f_params[:delete_id].to_i
      SidfmdRule.find(delete_id).destroy
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '削除に失敗しました' } and return 
    end
    Rails.cache.clear
    render :layout => false, :json => { :err_message => '' }
  end

  def bulk_delete
    begin 
      f_params = params.permit(:delete_ids)
      delete_ids = JSON.parse(f_params[:delete_ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です' } and return
    end
    begin 
      SidfmdRule.transaction do 
        SidfmdRule.where(:id => delete_ids).destroy_all
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '削除に失敗しました' } and return
    end 
    Rails.cache.clear
    render :layout => false, :json => { :err_message => '' }
  end

  def bulk_update_rule_flg
    f_params = params.permit(:ids, :flg)
    begin
      ids = JSON.parse(f_params[:ids])
    rescue JSON::ParserError => e
      render :layout => false, :json => { :err_message => '不正な操作です' } and return
    end
    flg = f_params[:flg] == 'enable' ? true : false
    begin 
      rules = SidfmdRule.where(:id => ids)
                .where.not(:flg_enable => flg)
      ids = rules.pluck(:id) # ログの更新のため、変更のあるルールのidのみに絞り、変更のないものはログを作らない
      rules.update_all(:flg_enable => flg)
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '変更に失敗しました' } and return
    end
    begin
      SidfmdRule.where(:id => ids).each do |rule|
        begin
          rule.with_lock do
            create_updated_rule_logs(rule, is_updated_status: true, is_updated_settings: false)
          end
        rescue Exception => e
          log_exception(e)
        end
      end
    rescue Exception => e
      log_exception(e)
    end
    Rails.cache.clear
  
    render :layout => false, :json => { :err_message => '' } 
  end

  def api_get_display_summary_content
    GC.start
    f_params = params.fetch(:pages, {}).permit(:per_page)
    parse_params
    check_filters
    @rules[:per_page] = f_params[:per_page].to_i

    if @sidfm_roles.include?(:admin) then
      @all_rules = SidfmdRule.all
    else
      @all_rules = SidfmdRule.where(:sidfmd_category_id => @user_custom_category_ids)
    end
  
    @all_sidfm_categories = SidfmdCategory.where(:is_system => false)
 
    search_items
    sort_rules
    @rules_info = []
    @rules[:content] = []
    @rule_ids = []
    user = User.current
    @all_rules.each_with_index do |rule, idx|
      rule_info = create_rule_info(rule)
      @rules_info << { :rule_id => rule.id, :param => rule_info[:param].dup }
      @rule_ids << rule.id
      if idx >= @rules[:offset] && idx < (@rules[:offset] + @rules[:per_page]) then
        @rules[:content] << create_summary_rule_html(
          rule_info,
          {
            :user => user,
            :checked =>  @rules[:checked],
          }
        )
      end
    end

    ret = {
      :rows => @rules[:content],
      :total => @rules_info.size,
      :all_ids => @rule_ids,
      :all_rule_count => SidfmdRule.count,
      :hidden_params => @rules_info,
      :sidfm_roles => @sidfm_roles,
      :user_cats => (@sidfm_roles.include?(:admin) ? nil : @user_custom_category_ids)
    }

    render :layout => false, :json => ret
  end
 
end  


