#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmCtagsHelper
  # サマリー表のチェックボックス
  def create_summary_ctag_chkbox(ctag_id, checked, disabled)
    content = check_box_tag(
      nil,
      ctag_id,
      checked.include?(ctag_id),
      :class => disabled ? '' : 'ctagSelect',
      :disabled => disabled
    )
  end

  # サマリー表の優先度
  def create_summary_ctag_priority(priority)
    content = content_tag(
      :div,
      "#{priority}"
    )
  end

  # サマリー表のタグ名
  def create_summary_ctag_name(name)
    content = content_tag(
      :div,
      "#{name}",
      :class => 'text-left'
    )
  end

  # サマリー表のカテゴリ
  def create_summary_ctag_category(category_name, category_desc)
    is_category_name = category_name && category_name.length > 0
    content = content_tag(
      :div,
      "#{is_category_name ? category_name : '全体'}",
      :class => "summary-category-name text-left word-break-all",
      :data => {
        :toggle => 'tooltip',
        :title => "#{category_desc && category_desc.length ? category_desc : ''}",
        :placement => 'top',
        :boundary => 'window'
      }
    )
  end

  # サマリー表の色
  def create_summary_ctag_color(text_color, bg_color)
    text_color_elm = content_tag(
      :div,
      :class => 'd-inline-block'
    ) do
      concat(
        content_tag(
          :div,
          :class => 'd-flex justify-content-center align-items-center pt-1 pb-1'
        ) do
          concat(
            content_tag(:span, '文字色', :class => 'font-weight-bold text-nowrap')
          )
          .concat(
            ((text_color && text_color.length > 0 ) ?
              content_tag(
                :span,
                nil,
                :class => 'ml-2 summary-ctag-color',
                :style => "background-color: #{text_color};",
                :data => { :toggle => 'tooltip', :title => "#{text_color}", :placement => 'right' }
              ) :
              content_tag(:span, '---', :class => 'ml-2 summary-ctag-color')
            )
          )
        end
      )
    end

    bg_color_elm = content_tag(
      :div,
      :class => 'd-inline-block ml-4'
    ) do
      concat(
        content_tag(
          :div,
          :class => 'd-flex justify-content-center align-items-center pt-1 pb-1'
        ) do
          concat(
            content_tag(:span, '背景色', :class => 'font-weight-bold text-nowrap')
          )
          .concat(
            ((bg_color && bg_color.length > 0 ) ?
              content_tag(
                :span,
                nil,
                :class => 'ml-2 summary-ctag-color',
                :style => "background-color: #{bg_color};",
                :data => { :toggle => 'tooltip', :title => "#{bg_color}", :placement => 'right' }
              ) :
              content_tag(:span, '---', :class => 'ml-2 summary-ctag-color')
            )
          )
        end
      )
    end

    content = content_tag(
      :div,
      :class => 'd-flex pl-1 pr-1 justify-content-center'
    ) do
      concat(text_color_elm)
      .concat(bg_color_elm)
    end
  end

  # サマリー表の操作
  def create_summary_ctag_edit(ctag, disabled)
    edit_button = disabled ?
      button_tag(
        '編集',
        :type => 'button',
        :class => 'btn btn-sm btn-edit mr-1 text-nowrap',
        :disabled => 'disabled'
      ) :
      button_tag(
        '編集',
        :type => 'button',
        :class => 'btn btn-sm btn-edit mr-1 text-nowrap',
        :data => { :ctag => ctag.to_json, :edit => 'true' }
      )
    del_button = disabled ?
      button_tag(
        '削除',
        :type => 'button',
        :class => 'btn btn-sm btn-danger text-nowrap',
        :disabled => 'disabled'
      ) :
      button_tag(
        '削除',
        :type => 'button',
        :class => 'btn btn-sm btn-danger text-nowrap',
        :data => { :ctagid => ctag.id, :name => ctag.name, :delete => 'true' }
      )
    content = content_tag(
      :div,
      :class => 'd-flex justify-content-center'
    ) do
      concat(
        edit_button
      )
      .concat(
        del_button
      )
    end
  end

end
