#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdCatGroup < ActiveRecord::Base
  has_and_belongs_to_many :users
  has_and_belongs_to_many :sidfmd_categories
  has_many :sidfmd_cat_groups_users
  has_many :sidfmd_cat_groups_categories

  #グループがユーザのグループかどうかを返す
  def is_user_group?(role_admin:, user_id:)
    ret = true
    unless role_admin then
      ret = self.sidfmd_cat_groups_users.where(:user_id => user_id).exists?
    end
    ret
  end

  def update_categories(role_admin:, user_category_ids:, user_id:, category_ids:)
    if !role_admin && !user_category_ids then
      raise ArgumentError, "must be user_category_ids"
    end
    category_ids = category_ids.map(&:to_i)
    unless role_admin then
      category_ids &= user_category_ids
    end
    self.sidfmd_categories = SidfmdCategory.where(:id => category_ids).where(:is_system => false)
  end

  # 指定したカテゴリ(id)のみが設定されているグループを返す
  # カテゴリ削除時に使用
  def self.only_categories(category_ids)
    category_ids = category_ids.map(&:to_i)
    ret = self.where(
      :id => SidfmdCatGroupsCategory.where(:sidfmd_category_id => category_ids).select(:sidfmd_cat_group_id)
    ).select(
      :id,
      :name,
      "ARRAY(
        SELECT sidfmd_category_id FROM sidfmd_cat_groups_categories
        WHERE sidfmd_cat_groups_categories.sidfmd_cat_group_id = sidfmd_cat_groups.id
      ) AS group_category_ids"
    ).map{|grp|
      (grp.group_category_ids - category_ids).size == 0 ? { :id => grp.id, :name => grp.name } : nil
    }.compact
  end

  # 指定したカテゴリグループに属するユーザとその全カテゴリを返す
  # カテゴリグループ削除時・変更時のルールチェックに使用
  # change_category_idsを指定すると、この値を指定したグループのカテゴリとして扱う
  def self.user_cat_group_categories(cat_group_ids:, change_category_ids: nil)
    cat_group_ids = cat_group_ids.map(&:to_i)
    change_category_ids = change_category_ids.map(&:to_i) if change_category_ids

    user_category_ids = {}
    add_each_data = ->(data){
      data.each do |data|
        user_category_ids[data.user_id] = Set.new unless user_category_ids[data.user_id]
        user_category_ids[data.user_id].add(data.sidfmd_category_id)
      end
    }

    cat_groups = self.joins(:sidfmd_cat_groups_users).joins("
      LEFT OUTER JOIN sidfmd_cat_groups_categories ON sidfmd_cat_groups.id = sidfmd_cat_groups_categories.sidfmd_cat_group_id
    ")
    .where( #指定したグループが設定されているユーザに紐づくグループを取得
      :sidfmd_cat_groups_users => {
        :user_id => SidfmdCatGroupsUser.where(
          :sidfmd_cat_group_id => cat_group_ids
        ).select(:user_id)
      }
    )

    user_ids = cat_groups.distinct.pluck(:user_id)

    if change_category_ids then
      user_ids.each do |user_id|
        user_category_ids[user_id] = change_category_ids.to_set       
      end
      add_each_data.( #指定されたグループ以外のグループのカテゴリ
        cat_groups.where.not(:sidfmd_cat_groups => { :id => cat_group_ids }).select(
          :user_id,
          :sidfmd_category_id
        )
      )
    else
      add_each_data.(
        cat_groups.select(
          :user_id,
          :sidfmd_category_id
        )
      )
    end

    add_each_data.(SidfmdCategoriesUser.where(:user_id => user_ids))

    user_category_ids.each do |user_id, cats|
      user_category_ids[user_id] = cats.to_a.compact
    end

    user_category_ids
  end

  # 指定したユーザのカテゴリグループのうち、指定したユーザのグループを除くグループのカテゴリを返す
  # 今の所、管理者以外でユーザを操作する際に使用
  def self.limit_user_group_category_ids(user_id, except_user_id)
    SidfmdCatGroupsUser.joins(:sidfmd_cat_group).joins("
      LEFT OUTER JOIN sidfmd_cat_groups_categories ON sidfmd_cat_groups.id = sidfmd_cat_groups_categories.sidfmd_cat_group_id
    ")
    .where.not(
      :sidfmd_cat_groups => {
        :id => SidfmdCatGroupsUser.where(:user_id => except_user_id).select(:sidfmd_cat_group_id)
      }
    )
    .where(:sidfmd_cat_groups_users => { :user_id => user_id } )
    .pluck(:sidfmd_category_id).compact
  end

end
