#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdLproduct < ActiveRecord::Base
  scope :support_end_products, -> { where("eol_date < ?", Date.today) }
  scope :name_matched, -> (keyword) {
    where(
      'LOWER(name) LIKE ? OR LOWER(name) LIKE ?',
      ActiveRecord::Base.send(:sanitize_sql_like, keyword.downcase) + '%',
      '% ' + ActiveRecord::Base.send(:sanitize_sql_like, keyword.downcase) + '%'
    )
  }

  has_many :sidfmd_lversions

  RELEASE_DATE_LABEL = 'リリース日付'
  SUPPORT_DATE_LABEL = '取扱開始日付'
  EOL_DATE_LABEL = 'EOL日付'

  def is_support_end
    eol_date ? Date.today > eol_date : false
  end

  def self.date_to_string(date)
    date.strftime('%Y/%m/%d')
  end

  # 変更によって削除されるバージョンidを返す
  # { プロダクトid => 変更後のversion_ids }のデータを渡す
  def self.get_delete_lversion_ids(lproduct_lversion_ids)
    current_lproduct_versions = {}
    self.joins(:sidfmd_lversions)
        .where(:id => lproduct_lversion_ids.keys)
        .select('sidfmd_lproducts.id AS lp_id', 'sidfmd_lversions.id AS lv_id')
        .each do |lp|
          current_lproduct_versions[lp.lp_id] = [] unless current_lproduct_versions[lp.lp_id]
          current_lproduct_versions[lp.lp_id] << lp.lv_id
        end
    del_version_ids = []
    lproduct_lversion_ids.each do |lproduct_id, version_ids|
      if current_lproduct_versions[lproduct_id.to_i] then
        del_version_ids += current_lproduct_versions[lproduct_id.to_i] - version_ids.map(&:to_i)
      end
    end
    del_version_ids
  end

end
