#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdQueueDist < ActiveRecord::Base
  belongs_to :sidfmd_queue

  scope :enable_hosts, -> { joins(:sidfmd_queue).where(:sidfmd_queues => { :sidfmd_host_id => SidfmdHost.enable_hosts.categorized.select(:id) }) }

  # 有効なホストのident, version, nameversionを取得可能。
  def self.get_dist_versions
    self.enable_hosts.select('ident,version,CONCAT(CONCAT(name,\' \'),version) AS nameversion').distinct('nameversion')
  end

  # OS毎の分布を返す
  # FIXME: 今のところAgent経由のみ
  def self.distribution_host(category_id = 0)
    distversions = SidfmdHost.enable_hosts.categorized
    if category_id > 0 then
      distversions = distversions.joins(:sidfmd_categories_hosts).where(:sidfmd_categories_hosts => { :sidfmd_category_id => category_id })
    end
    distversions = distversions.select(:id,'(SELECT CONCAT(name,\' \',version) FROM sidfmd_queue_dists WHERE sidfmd_queue_id=(SELECT id FROM sidfmd_queues WHERE sidfmd_host_id=sidfmd_hosts.id ORDER BY created_at DESC LIMIT 1) ORDER BY created_at DESC LIMIT 1) AS distversion')
    ret = {}
    distversions.each do |distversion|
      if ret.has_key?(distversion[:distversion]) then
        ret[distversion[:distversion]] += 1
      else
        ret[distversion[:distversion]] = 1
      end
    end
    ret
  end

  def get_nameversion
    "#{name} #{version}"
  end

  #更新可能なパッケージのOSをDBに登録する際のデータを返す
  def self.get_updatable_pkg_os_data(osname)
    ret = {}
    if /Amazon Linux (\d+\.*\d*)/ =~ osname then
      version = Regexp.last_match(1)
      major_version = version.slice(/\d+/)
      if SidfmdQueuePkg::AMAZON_LINUX_LIST.include?("Amazon Linux #{major_version}") then
        ret[:id] = 'amzn'
        ret[:os] = 'Amazon Linux'
        ret[:version] = version
      end
    elsif /AlmaLinux (\d+\.*\d*)/ =~ osname then
      version = Regexp.last_match(1)
      major_version = version.slice(/\d+/)
      if SidfmdQueuePkg::ALMA_LINUX_LIST.include?("AlmaLinux #{major_version}") then
        ret[:id] = 'almalinux'
        ret[:os] = 'AlmaLinux'
        ret[:version] = version
      end
    elsif /CentOS (\d+\.*\d*)/ =~ osname then
      version = Regexp.last_match(1)
      major_version = version.slice(/\d+/)
      if SidfmdQueuePkg::CENTOS_LIST.include?("CentOS #{major_version}") then
        ret[:id] = 'centos'
        ret[:os] = 'CentOS'
        ret[:version] = version
      end
    elsif /Debian GNU\/Linux (\d+\.*\d*)/ =~ osname or /Debian (\d+\.*\d*)/ =~ osname then
      version = Regexp.last_match(1)
      major_version = version.slice(/\d+/)
      if SidfmdQueuePkg::DEBIAN_LIST.include?("Debian GNU/Linux #{major_version}.0") then
        ret[:id] = 'debian'
        ret[:os] = 'Debian'
        ret[:version] = version
      end
    elsif /Red(\s{1}|)Hat Enterprise Linux (\d+\.*\d*)/ =~ osname then
      version = Regexp.last_match(2)
      major_version = version.slice(/\d+/)
      if SidfmdQueuePkg::RHEL_LIST.include?("Red Hat Enterprise Linux #{major_version}") then
        ret[:id] = 'rhel'
        ret[:os] = 'Red Hat Enterprise Linux'
        ret[:version] = version
      end
    elsif /Ubuntu (\d+\.\d+)/ =~ osname then
      version = Regexp.last_match(1)
      if SidfmdQueuePkg::UBUNTU_LIST.include?("Ubuntu #{version} LTS") then
        ret[:id] = 'ubuntu'
        ret[:os] = 'Ubuntu'
        ret[:version] = Regexp.last_match(1)
      end
    end

    if ret.size == 0 then
      raise ArgumentError#"osname is invalid"
    end

    ret 
  end

end
