/* coding: utf-8 */

//BootStrapTableのtdのスタイル。画面ごとに関数を作って設定する
function BootStrapTableCellStyle(value, row, index, field) {
    switch (field){
        case 'chkbox':
            return { classes: 'chk-box' };
        case 'id':
            return { classes: 'id' };
        case 'name':
            return { classes: 'name' };
        case 'categories':
            return { classes: 'categories' };
        case 'edit':
            return { classes: 'edit' };
        default:
            return { classes: '' };
    }
}

$(function() {
    activate_sidebar_item('sidebar-item-catgroup-manip');

    const summaryTableId = 'cat-groups-summary-table';
    const summaryTableElm = $('#' + summaryTableId);
    const pageName = 'cat-groups';

    //グループ削除時のルールチェック
    function checkDeleteGroupRelatedRules(ids){
        return new Promise((resolve, reject) => {
            $.ajax({
                type: 'POST',
                url: $('#sidfm_internal_api_check_related_rules_path').val(),
                data: { page_name: 'cat_group', action_idx: 'delete', delete_ids: JSON.stringify(ids) }
            }).done(function(data){
                resolve(data);
            }).fail(function(){
                alert('エラーが発生したため、削除に失敗しました。');
                $('#pages_form').submit();
            });
        });
    }

    //カテゴリ変更時のルールチェック
    //引数には、{ グループid: 変更後のカテゴリids }を渡す
    function checkCategoryChangeGroupRelatedRules(cat_group_category_ids_hash){
        return new Promise((resolve, reject) => {
            $.ajax({
                type: 'POST',
                url: $('#sidfm_internal_api_check_related_rules_path').val(),
                data: {
                    page_name: 'cat_group',
                    action_idx: 'category_change',
                    cat_group_category_ids: JSON.stringify(cat_group_category_ids_hash)
                }
            }).done(function(data){
                resolve(data);
            }).fail(function(){
                alert('エラーが発生したため、変更に失敗しました。');
                $('#pages_form').submit();
            });
        });
    }

    //変更できないルールがあるときに、アラート表示
    function showUnchangeableRulesAlert(){
        setTimeout(function(){
            alert(
                'ログインユーザのカテゴリ範囲外のルールの条件、操作もしくはルール自体が削除されるため、処理を実行をすることができません。\n' +
                '※処理実行がどうしても必要な場合は、SIDfmVM管理者権限のユーザで、再度実行してください。'
            );
       },100);
    }

    //新規登録
    const registNameInput = $('#group-name-input');
    const registSelectedCategoriesElm = $('#regist-selected-categories');
    const registUnselectedCategoriesElm = $('#regist-unselected-categories');
    const registSelectedCategoryIdsElm = $('#regist-selected-category-ids');
    const registModifyIdElm = $('#regist-modify-id');
    const userCategories = JSON.parse($('#selectable-categories').val());
    const registModalTitleElm = $('#modalRegist .modal-title');

    $('#modalRegist').on('hide.bs.modal', function(){
        registSelectedCategoriesElm.scrollTop(0);
        registUnselectedCategoriesElm.scrollTop(0);
    });

    $('#regist-select-category').on('click', function(){
        const selectedCategoryIds = JSON.parse(registSelectedCategoryIdsElm.val());
        const selectedCategories = registUnselectedCategoriesElm.find('option:selected');
        selectedCategories.each((idx, elm) => {
            selectedCategoryIds.push(parseInt($(elm).val()));
        });
        registSelectedCategoriesElm.append(selectedCategories);
        registSelectedCategoryIdsElm.val(JSON.stringify(selectedCategoryIds));
        toggleRegistButton();
    });

    $('#regist-unselect-category').on('click', function(){
        let selectedCategoryIds = JSON.parse(registSelectedCategoryIdsElm.val());
        const removeCategoryIds = [];
        const unselectCategories = registSelectedCategoriesElm.find('option:selected');
        unselectCategories.each((idx, elm) => {
            removeCategoryIds.push(parseInt($(elm).val()));
        });
        selectedCategoryIds = selectedCategoryIds.filter(selectedCatId => !removeCategoryIds.includes(selectedCatId));
        registUnselectedCategoriesElm.append(unselectCategories);
        registSelectedCategoryIdsElm.val(JSON.stringify(selectedCategoryIds));
        toggleRegistButton();
    })

    function resetRegistModal(){
        registModifyIdElm.val('');
        registNameInput.val('');
        registSelectedCategoryIdsElm.val('[]');
        registSelectedCategoriesElm.find('option').remove();
        registUnselectedCategoriesElm.find('option').remove();
        toggleRegistButton();
    }

    function createRegistCategoryOption(category){
        return $('<option>').attr({
            'data-toggle': 'tooltip',
            'data-title': category.desc,
            'data-boundary': 'window',
            'data-placement': 'right'
        }).val(category.id).text(category.name);
    }

    function bindRegistCategoryOptionEvent(){
        registSelectedCategoriesElm.find('[data-toggle="tooltip"]').tooltip();
        registUnselectedCategoriesElm.find('[data-toggle="tooltip"]').tooltip();
    }

    function toggleRegistButton(){
        let isDisabled = false;
        if( !registNameInput.val() ) isDisabled = true;
        if( !isDisabled ){
            const selectedCategoryIds = JSON.parse(registSelectedCategoryIdsElm.val());
            if( !selectedCategoryIds.length ) isDisabled = true;
        }
        $('#exec-regist').prop('disabled', isDisabled);
    }

    registNameInput.on('input', function(){
        toggleRegistButton();
    });

    $('#create-new-group-btn').on('click', function(){
        resetRegistModal();
        registModalTitleElm.text('新規登録');
        $('#exec-regist').text('登録');
        registUnselectedCategoriesElm.append(
            userCategories.map(cat => createRegistCategoryOption(cat))
        );
        bindRegistCategoryOptionEvent();
        $('#regist-action-path').val($('#create-group-path').val());
        $('#modalRegist').modal('show');
    });

    $('#exec-regist').on('click', function(){
        const button = $(this);
        button.addClass('pointer-events-none');
        const close = () => {
            button.removeClass('pointer-events-none');
        };
        (async() => {
            const paramsData = {
                name: registNameInput.val(),
                category_ids: registSelectedCategoryIdsElm.val()
            };
            if( registModifyIdElm.val() ) { //変更時
                paramsData.id = registModifyIdElm.val();
                const rulesData = await checkCategoryChangeGroupRelatedRules({
                    [paramsData.id]: JSON.parse(paramsData.category_ids)
                });
                if( rulesData['is_unchangeable_user'] ){ //カテゴリ変更なので、通常ここには入らない
                    showUnchangeableRulesAlert(); 
                    close();
                    return false;
                }else{
                    if(Object.keys(rulesData['rule_messages']).length > 0){
                        $('#submit-related-rules-item-form').attr({'data-actionname': 'category_change', 'data-data': paramsData.id});
                        close();
                        display_related_rules(rulesData);
                        return;
                    }
                    close();
                    dynamicSummaryTable.checkRolesAndCatsFlgElm.val('false');
                }
            }
            dynamicSummaryTable.executeProcess({
                paramsData,
                path: $('#regist-action-path').val(),
                additionalAlertMessage: '登録に失敗しました。'
            });
            close();
        })();
    });

    function createConfUserElm(user){
        return $('<tr>').append(
            $('<td>').text(user.login),
            $('<td>').text(`${user.lastname} ${user.firstname}`)
        );
    }

    function showConfUsersModal(actionName, data, users){
        const descElm = $('#modalConfirmUsersDesc');
        if( actionName === 'delete' || actionName === 'bulk_delete'){
            descElm.html('以下のユーザに削除対象のカテゴリグループが設定されています。<br>' +
                'カテゴリグループを削除しても問題が無い場合は、OKボタンをクリックして下さい。'
            );
        }else{
            descElm.text('');
        }
        const tbody = $('#modalConfirmUsersTbody');
        tbody.html('');
        tbody.append(
            users.map(user => createConfUserElm(user))
        );
        $('#exec-after-confirm-users').attr({'data-actionname': actionName, 'data-data': data});
        $('#modalConfirmUsers').modal('show');
    }

    $('#exec-after-confirm-users').on('click', function(){
        const button = $(this);
        button.addClass('pointer-events-none');
        const actionName = button.attr('data-actionname');
        const data = button.attr('data-data');
        const close = () => {
            button.removeClass('pointer-events-none');
            $('#checking-window').remove();
        };
        $('body').append(create_loading_window()); 
        (async () => {
            dynamicSummaryTable.checkRolesAndCatsFlgElm.val('false'); 
            if( actionName === 'delete' ){
                const rulesData = await checkDeleteGroupRelatedRules([data]);
                if( rulesData['is_unchangeable_user'] ){
                    showUnchangeableRulesAlert();
                    close();
                    return;
                }else{
                    if(Object.keys(rulesData['rule_messages']).length > 0){
                        $('#submit-related-rules-item-form').attr({'data-actionname': actionName, 'data-data': data});
                        close();
                        display_related_rules(rulesData);
                        return;
                    }
                    dynamicSummaryTable.executeProcess({
                        paramsData: { id: data },
                        path: $('#delete-group-path').val(),
                       additionalAlertMessage: '削除に失敗しました。'
                    });
                }
            }else if( actionName === 'bulk_delete' ){
                const rulesData = await checkDeleteGroupRelatedRules(JSON.parse(data));
                if( rulesData['is_unchangeable_user'] ){
                    showUnchangeableRulesAlert();
                    close();
                    return;
                }else{
                    if(Object.keys(rulesData['rule_messages']).length > 0){
                        $('#submit-related-rules-item-form').attr({'data-actionname': actionName, 'data-data': data});
                        close();
                        display_related_rules(rulesData);
                        return;
                    }
                    dynamicSummaryTable.executeProcessWithProgressBar({
                        ids: JSON.parse(data),
                        paramsData: {},
                        idsParamsName: 'ids',
                        path: $('#bulk-delete-group-path').val(),
                    });
                }
            }
            close();
        })();
    });

    $('#submit-related-rules-item-form').on('click', function(){
        const button = $(this);
        button.addClass('pointer-events-none');
        const actionName = button.attr('data-actionname');
        const data = button.attr('data-data');
        const close = () => {
            button.removeClass('pointer-events-none');
            $('#checking-window').remove();
        };
        $('body').append(create_loading_window());
        dynamicSummaryTable.checkRolesAndCatsFlgElm.val('false');
        if( actionName === 'delete' ){
            dynamicSummaryTable.executeProcess({
                paramsData: { id: data },
                path: $('#delete-group-path').val(),
                additionalAlertMessage: '削除に失敗しました。'
            });
        }else if( actionName === 'bulk_delete' ){
            dynamicSummaryTable.executeProcessWithProgressBar({
                ids: JSON.parse(data),
                paramsData: {},
                idsParamsName: 'ids',
                path: $('#bulk-delete-group-path').val(),
            });
        }else if( actionName === 'category_change' ){
            dynamicSummaryTable.executeProcess({
                paramsData: {
                    id: registModifyIdElm.val(),
                    name: registNameInput.val(),
                    category_ids: registSelectedCategoryIdsElm.val(),
                },
                path: $('#regist-action-path').val(),
                additionalAlertMessage: '登録に失敗しました。'
            });
        }
        close();
    });
            
    //編集
    const enableEditGroup = () => {
        $('#' + summaryTableId + ' [data-edit="true"]').on('click', function(){
            resetRegistModal();
            registModalTitleElm.text('カテゴリグループの編集');
            $('#exec-regist').text('更新');
            $('#regist-action-path').val($('#modify-group-path').val());
            const button = $(this);
            const id = button.attr('data-groupid');
            registModifyIdElm.val(id);
            registNameInput.val(button.attr('data-name'));
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
                if( roleOrCategoryError ) return;
                $.post($('#get-group-categories-path').val(), { id } , function(cats) {
                    const groupCatElms = [];
                    const notGroupCatElms = [];
                    const categoriesHash = {};
                    cats.forEach(cat => categoriesHash[cat.id] = cat);
                    userCategories.forEach(cat => {
                        if( categoriesHash[cat.id] ){
                            groupCatElms.push(createRegistCategoryOption(cat));
                        }else{
                            notGroupCatElms.push(createRegistCategoryOption(cat));
                        }
                    });
                    registSelectedCategoriesElm.html('');
                    registUnselectedCategoriesElm.html('');

                    registSelectedCategoriesElm.append(groupCatElms);
                    registUnselectedCategoriesElm.append(notGroupCatElms);
                    registSelectedCategoryIdsElm.val(JSON.stringify(cats.map(cat => cat.id)));
                    bindRegistCategoryOptionEvent();
                    toggleRegistButton();
                    $('#modalRegist').modal('show');
                });
            })();
        });
    };

    //削除
    const enableDeleteGroup = () => {
        $('#' + summaryTableId + ' [data-delete="true"]').on('click', function(){
            const button = $(this);
            button.addClass('pointer-events-none');
            const id = button.attr('data-groupid');
            const confText = '「' + button.attr('data-name') + ' 」を削除してもよろしいですか?';
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
                if( roleOrCategoryError ) return;
                $.post(
                    $('#api-get-cat-group-related-users-path').val(),
                    { ids: JSON.stringify([id]) },
                    function(users){
                        if( !users.length ) {
                            let conf = confirm(confText);
                            if( !conf ) {
                                button.removeClass('pointer-events-none');
                                return;
                            }
                            dynamicSummaryTable.executeProcess({
                                paramsData: { id },
                                path: $('#delete-group-path').val(),
                                additionalAlertMessage: '削除に失敗しました。'
                            });
                        }else{
                            showConfUsersModal('delete', id, users);
                        }
                        button.removeClass('pointer-events-none');
                    }
                );
            })();
        });
    };

    adjust_balloon_dropdown_style();

    //クリックでカテゴリ表示
    const createGroupCategoryListItem = (category, filterElmId) => {
        return $('<a>').attr({
            'class': 'categories-list-item list-group-item text-truncate g-max-width-16em',
            'href': '#',
            'data-toggle': 'tooltip',
            'data-title': category.desc,
            'data-placement': 'right',
            'data-boundary': 'window',
            'data-filteritem': 'category_id',
            'data-value': category.id,
            'data-name': 'category_id',
            'data-filterid': filterElmId
        }).text(category.name);
    };

    const enableDisplayGroupCategories = () => {
        let jqxhr = null;
        $(`#${summaryTableId} .show-group-categories`).on('click', function(){
            const triggerBtn = $(this);
            const id = triggerBtn.attr('data-groupid');
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
                if( roleOrCategoryError ) return;
                if( triggerBtn.attr('isdata') === 'true' ){
                    return;
                }
                triggerBtn.attr('isdata', 'true');
                const triggerParent = triggerBtn.closest('.balloon-dropdown');
                const maxWidth = triggerParent.find('.balloon-menu').css('max-width').replace('px', '');
                const isSpaceOfRight = $(window).width() - (triggerBtn.offset().left + triggerBtn.outerWidth()) > maxWidth;
                triggerParent.removeClass('dropright dropleft balloon-up balloon-right balloon-left')
                             .addClass(isSpaceOfRight ? 'dropright balloon-right' : 'dropleft balloon-left');
                if( jqxhr ){
                    jqxhr.abort();
                }
                jqxhr = $.post(
                    $('#get-group-categories-path').val(),
                    { id },
                    function(cats) {
                        if( cats.length === 0 ){
                            alert('データが変更されているため、画面をリロードします。')
                            $('#pages_checked').val('[]');
                            $('#pages_all_checked').val('false');
                            $('#pages_form').submit();
                        }
                        const listMenu = triggerParent.find('[data-itemlist="true"]');
                        const elmIds = [];
                        cats.slice(0, 15).forEach(cat => {
                            const elmId = String(cat.id) + 'group-' + id;
                            elmIds.push(elmId);
                            listMenu.append(createGroupCategoryListItem(cat, elmId));
                            summaryTableFilter.enableFilterCategoryLink('[data-filterid="' + elmId + '"]', 'category_id', 'single');
                        });
                        listMenu.find('[data-toggle="tooltip"]').tooltip();
                        setTimeout(function(){
                            cats.slice(15).forEach(cat => {//残りを作成
                                const elmId = String(cat.id) + 'group-' + id;
                                listMenu.append(createGroupCategoryListItem(cat, elmId));
                                summaryTableFilter.enableFilterCategoryLink('[data-filterid="' + elmId + '"]', 'category_id', 'single');
                            });
                            listMenu.find('[data-toggle="tooltip"]').tooltip();
                        });
                        triggerBtn.attr({
                            'data-toggle': 'dropdown',
                            'data-boundary': 'window',
                            'aria-haspopup': 'true',
                            'aria-expanded': 'false',
                            'data-flip': 'false',
                        }).dropdown();
                        if( !listMenu.is(':visible') ){
                            triggerBtn.click();
                        }
                    }
                );

            })();
        });
    };

    //カテゴリ名(単数)を押下した時のカテゴリ検索
    const enableFilterCategoryLinks = (selector) => {
        summaryTableFilter.enableFilterCategoryLink(selector, 'category_id', 'single');
    };

    //サマリー表の動的処理のため(bootstrap-tableのため)にbindする。でないとイベントが効かない。
    const bindSummaryTableRowEvents = () => {
        enable_bootstrap_actions('#' + summaryTableId);
        enableEditGroup();
        enableDeleteGroup();
        enableDisplayGroupCategories();
        enableFilterCategoryLinks('[data-filteritem="category_id"]');
    };

    const dynamicSummaryTable = new DynamicSummaryTable({
        summaryTableId,
        bindSummaryTableRowEventsFunction: bindSummaryTableRowEvents,
        pageName,
        allFilteredIdsId: 'all-group-ids',
        chkboxSelector: `#${summaryTableId} .groupSelect`,
        allChkboxSelector: `#${summaryTableId} .allGroupSelect`,
        bulkBtnSelector: '.action-btn',
        checkRolesAndCatsFlgElm: $('#check-roles-cats-flg'),
        checkUserInfoTarget: 'auth'
    });

    set_style_for_ActionArea_on_filter_dropdown_event();
    sticky_control('SummaryTableButtonsArea', pageName);
    $('#pages_form').on('submit', function(){ set_summary_table_lines(pageName, $('#pages_per_page').val()) });

    //一括削除
    const createBulkTrElm = (name) => {
        let tr = $('<tr>');
        tr.append(
            $('<td>').attr('class', 'text-left text-break').text(name)
        );
        return tr;
    };

    $('#modalBulkDelete').on('hide.bs.modal', function(){
        $('#modalBulkDeleteTbody').closest('.modal-mail-table').scrollTop(0);
    });

    $('#bulk-delete-groups').on('click', function(){
        let checked = JSON.parse($('#pages_checked').val());
        $('#bulk-delete-ids').val(JSON.stringify(checked));
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '削除に失敗しました。'
            });
            if( roleOrCategoryError ) return;
            $.post( $('#api-get-cat-group-names-path').val(), { ids: JSON.stringify(checked) }, function(names){
                $('#modalBulkDeleteTbody *').remove();
                const sliceNum = 20;
                let nameElms = [];
                const target = $('#modalBulkDeleteTbody');
                names.slice(0, sliceNum).forEach( name => nameElms.push(createBulkTrElm(name)) );
                target.append(nameElms);
                $('#modalBulkDelete').modal('show');
                setTimeout(function(){
                    names.slice(sliceNum).forEach( name => target.append(createBulkTrElm(name)) );
                });
            });
        })();
    });

    $('#exec-bulk-delete').on('click', function(){
        const confText = '一括削除します。よろしいですか?';
        const ids = JSON.parse($('#bulk-delete-ids').val());
        const button = $(this);
        button.addClass('pointer-events-none');
        const cancel = () => {
            button.removeClass('pointer-events-none');
            $('#checking-window').remove();
        };
        (async () => {
            $('body').append(create_loading_window());
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
            $.post(
                $('#api-get-cat-group-related-users-path').val(),
                { ids: JSON.stringify(ids) },
                function(users) {
                    if( !users.length ) {
                        let conf = confirm(confText);
                        if( !conf ) {
                            cancel();
                            return;
                        }
                        dynamicSummaryTable.executeProcessWithProgressBar({
                            ids,
                            paramsData: {},
                            idsParamsName: 'ids',
                            path: $('#bulk-delete-group-path').val(),
                        });
                    }else{
                        showConfUsersModal('bulk_delete', JSON.stringify(ids), users);
                    }
                    cancel();
                }
            );
        })();
    });

    //検索
    const summaryTableFilter = new SummaryTableFilter({dynamicSummaryTable});
    summaryTableFilter.enableFilterText(
        '#filter-name',
        '#filter-name-reset',
        { fieldSelector: '#filters-search-name' },
        'name'
    ); //グループ名検索
    summaryTableFilter.enableFilterChangeLink('.category-change'); //カテゴリ検索
    summaryTableFilter.enableResetAllFilters('#filter-clear'); //全リセット

    //JSON/TSV出力
    $('#download-files-btn').on('click', function(){
        $('#modalDownload').modal('show');
    });

    $('#exec-download').on('click', function(){
        json_tsv_summary_table_progress_download($('#download_select').val(), $('#read-download-data-path').val(), { dynamicSummaryTable });
    });
});
