/* coding: utf-8 * /
/* Copyright (C) 2021 SofTek Systems, Inc. */

$(function(){
    init_sidfm();
    enable_dropdown_balloon();

    $('.cancel-import').on('click', function(){
        $('#form-to-index-page').submit();
    });

    function get_delete_category_ids(){
        const actions = JSON.parse($('#actions').val());
        let delete_cat_ids = [];
        actions.forEach(function(action){
            if( action.action === 'delete' ){
                delete_cat_ids.push(action.cat_id);
            }
        });
        return delete_cat_ids;
    }

    function import_apply_with_progress_bar(){
        var import_apply_path = $('#import-apply-path').val();
        var actions = JSON.parse($('#actions').val());
        var progress_window = get_progress_bar();
        $('body').append(progress_window);
        var executed_actions = 0;
        var total_actions = actions.length;
        var err_messages = new Set();
        var err_cnt = 0;
        var cnt = 0;
  
        var send_data_of_import = function(){
            action = actions[cnt];
            action_data = { 'action_data': JSON.stringify(action) };
            if( cnt == 0 ){
                //初めにルールを処理する
                delete_category_ids = get_delete_category_ids();
                action_data['delete_category_ids'] = JSON.stringify(delete_category_ids);
                action_data['process_rules'] = true;
            }
            $.ajax({
                type: 'POST',
                url: import_apply_path,
                data: action_data,
            }).done(function(result){
                if( result['rules_error'] ){
                    return cancel_process_related_to_rules($('#index_path').val());
                }
                err_message = result['err_message'];
                if( err_message.length > 0 ){
                    err_messages.add(err_message);
                    err_cnt += 1;
                }
                executed_actions += 1;
                var percentage = (executed_actions / total_actions) * 100;
                if( percentage >= 99 && percentage < 100 ){
                    percentage = 99;
                }else{
                    percentage = Math.round(percentage);
                }
                $('#progress-bar').attr('aria-valuenow', percentage).css('width', percentage + '%').text(percentage + '%');
                if( percentage == 100 ){
                    setTimeout(function(){//progress barのbarが満たされるまで待つ
                        if(err_messages.size > 0){
                            err_messages = Array.from(err_messages);
                            alert(err_messages.join('\n') + (err_cnt == total_actions ? '' : '\n※他のデータの登録は完了しています。') + '\nカテゴリ設定画面に移動します。');
                            $('#form-to-index-page').submit();
                        }else{
                            $('#form-to-index-page').submit();
                        }
                    },800);
                }else{
                    cnt += 1;
                    send_data_of_import();
                }
            });
        };

        send_data_of_import();
    };
    
    $('.execute-submit-import-apply').on('click', function(){
        var del_cat_ids = get_delete_category_ids();
        if( del_cat_ids.length > 0 ){
            
            $('body').append(create_loading_window());

            var is_related_hosts = false;
            var is_related_users = false;
            var is_related_rules = false;

            var ctag_info = {};
            var fetch_ctag_info = $.post(
                $('#api-category-related-ctags-path').val(),
                { ids: JSON.stringify(del_cat_ids) },
                function(data){
                    ctag_info = data;
                }
            );
            var cat_group_info = {};
            var fetch_cat_group_info = $.post(
                $('#api-check-related-cat-groups-path').val(),
                { action_name: 'delete', ids: JSON.stringify(del_cat_ids) },
                function(data){
                    cat_group_info = data;
                }
            );

            var check_rules = $.ajax({
                url: $('#sidfm_internal_api_check_related_rules_path').val(),
                type: 'POST',
                data: { 'page_name': 'category_import', 'delete_category_ids': JSON.stringify(del_cat_ids), 'action_idx': 'import' },
            }).done(function(data){
                var rule_messages = data['rule_messages'];
                var rule_names = Object.keys(rule_messages);
                if(rule_names.length > 0){
                    is_related_rules = true;
                    display_related_rules(data, false);
                    $('#related-rule-info-area').show();
                }else{
                    $('#related-rule-info-area').hide();
                }
            });
            
            var check_hosts = $.ajax({
                url: $('#check_categories_related_hosts_path').val(),
                type: 'POST',
                data: { 'category_ids': JSON.stringify(del_cat_ids) }
            }).done(function(hosts){
                if( hosts.length > 0 ){
                    is_related_hosts = true;
                    $('#related-host-info-area-tbody > *').remove();
                    $('#related-host-info-area').show();
                    var host_elms = hosts.map(function(host){
                        return $('<tr>').attr('class', 'list_alternative').append(
                            $('<td>').append(
                                 $('<div>').attr({
                                     'data-toggle': 'tooltip',
                                     'data-title': ('ホスト名: ' + host.hostname + '<br/>表示名: ' + (host.ext_name ? host.ext_name : '(未設定)')),
                                     'data-html': 'true',
                                     'data-boundary': 'window',
                                     'data-placement': 'left',
                                 }).text(host.ext_name ? host.ext_name : host.hostname)
                            )
                        );
                    });
                    $('#related-host-info-area-tbody').append(host_elms);
                    $('#related-host-info-area-tbody').find('[data-toggle="tooltip"]').tooltip();
                }else{
                    $('#related-host-info-area').hide();
                } 
            });

            var check_users = $.ajax({
                url: $('#check_categories_related_users_path').val(),
                type: 'POST',
                data: { 'category_ids': JSON.stringify(del_cat_ids)} 
            }).done(function(users){
                if( users.length > 0 ){
                    is_related_users = true;
                    $('#related-user-info-area').show();
                    $('#related-user-info-area-tbody > *').remove();
                    var user_elms = users.map(function(user){
                        return $('<tr>').attr('class', 'list_alternative').append(
                                   $('<td>').append(
                                       $('<div>').text(user.login)
                                   ),
                                   $('<td>').append(
                                       $('<div>').text(user.lastname + ' ' + user.firstname)
                                   )
                               );
                    });
                    $('#related-user-info-area-tbody').append(user_elms);
                }else{
                    $('#related-user-info-area').hide();
                }
            });


            $.when(check_rules, check_hosts, check_users, fetch_ctag_info, fetch_cat_group_info).done(function(){
                $('#checking-window').remove();
                let conf = true;
                let cat_group_info_msg = '';
                if( cat_group_info.delete_cat_groups && cat_group_info.delete_cat_groups.length ){
                    cat_group_info_msg = '削除するカテゴリのみが設定されているカテゴリグループがあります。\n' +
                                         'カテゴリの削除を実行すると、そのカテゴリグループも削除されます。\n' +
                                         'よろしいですか?';
                }
                if( ctag_info.is_related_ctag && ctag_info.is_ctag_related_issue){
                    conf = confirm(
                        '削除するカテゴリに紐づくカスタムタグ、及びそのカスタムタグに紐づく脆弱性があります。\n' +
                        'インポートを実行すると、削除するカテゴリに紐づくカスタムタグ、及びカスタムタグと脆弱性の紐付きも削除されます。\n' +
                        (cat_group_info_msg ? ('また、' + cat_group_info_msg + '\n') : 'よろしいですか?')
                    );
                }else if( ctag_info.is_related_ctag ){
                    conf = confirm(
                        '削除するカテゴリに紐づくカスタムタグがあります。\n' +
                        'インポートを実行すると、削除するカテゴリに紐づくカスタムタグも削除されます。\n' +
                         (cat_group_info_msg ? ('また、' + cat_group_info_msg + '\n') : 'よろしいですか?')
                    );
                }else if(cat_group_info_msg){
                    conf = confirm(cat_group_info_msg);
                }
                if( !conf ){
                    return false;
                }
                if( is_related_rules || is_related_users || is_related_hosts){
                    $('#related-host-info-content, #related-user-info-content, #related-rule-info-content').hide();
                    $('#related-host-info-btn-icon, #related-user-info-btn-icon, #related-rule-info-btn-icon').removeClass('fa-angle-up')
                                                                                                              .addClass('fa-angle-down')
                    var title_items = [];
                    if( is_related_hosts ){
                        title_items.push('ホスト');
                    }
                    if( is_related_users ){
                        title_items.push('ユーザ');
                    }
                    if( is_related_rules ){
                        title_items.push('ルール');
                    }
                    $('#modalConfRelatedRules-title').text('削除対象のカテゴリに関連する' + title_items.join('/') + 'の情報');
                    $('#modalConfRelatedRules').modal('show');
                }else{
                    import_apply_with_progress_bar();   
                }
            });             
        }else{
            import_apply_with_progress_bar();
        }
    });

    $('#submit-related-rules-item-form').on('click', function(){
        import_apply_with_progress_bar();
    });
  
    function toggle_category_related_info_content(target_content_id, target_icon_id){
        var target_content = $('#' + target_content_id);
        var target_icon = $('#' + target_icon_id);
        if( target_content.is(':visible') ){
             target_content.hide();
             target_icon.removeClass('fa-angle-up').addClass('fa-angle-down');
        }else{
            target_content.show();
            target_icon.removeClass('fa-angle-down').addClass('fa-angle-up');
        }
    }

    $('#toggle-related-host-info-content-btn').on('click', function(){
        toggle_category_related_info_content('related-host-info-content', 'related-host-info-btn-icon');
    });

    $('#toggle-related-user-info-content-btn').on('click', function(){
        toggle_category_related_info_content('related-user-info-content', 'related-user-info-btn-icon');
    });

    $('#toggle-related-rule-info-content-btn').on('click', function(){
        toggle_category_related_info_content('related-rule-info-content', 'related-rule-info-btn-icon');
    });

    $('#modalConfRelatedRules').on('hide.bs.modal', function(){
        $(this).find('.Related-rule-table').scrollTop(0);
    });
});
