# -*- coding: utf-8 -*-

class CreateRegistrationKeys < ActiveRecord::Migration
  def change
    # エージェントでのホスト登録APIキー
    create_table :sidfmd_regkeys do |t|
      # 名前。一覧で区別する以外の用途無し。
      t.text :name, null: false

      # APIキー。実際にエージェントでのホスト登録で利用する。
      # nilは無効を意味する。
      # なお、ホスト登録以外（データ送信）では使用されない。
      t.text :api_key, default: nil, index: true

      # ホスト数制限
      # -1:無制限
      # 0以上: 設定されているカスタムカテゴリで指定した数のホストが
      # すでに登録されていた場合、登録に必ず失敗する。
      # ホスト数は登録方法(エージェント、手動、インポート）に影響しない。
      # 0は必ず登録に失敗する。
      t.integer :hostnum_limit, null: false, default: -1

      # ホスト名のprefix。必ずこの文字列が付与される。
      # ホスト名と同じ制限。
      t.text :hostname_prefix, null: false, default: ''

      # ホスト名のsuffix。必ずこの文字列が付与される。
      # ホスト名と同じ制限。
      t.text :hostname_suffix, null: false, default: ''

      # エージェントでのホスト登録で、既に同じホスト名が存在していた
      # 場合の挙動を指定
      # use: 既に登録済と判定して、登録済ホストの情報を返す。
      # ただし、カテゴリが一致していなければエラーを返す。
      # reject: 無条件で登録拒否
      # num_suffix: ホスト名(数値) で重複していない名前で登録する。
      # -数値で1から数値を付与していって、存在しないホスト名を探し
      # て登録。
      t.text :dup_action, null: false, default: 'reject'

      # 表示名にprefix/suffix適用前のホスト名を設定するか否か。
      # hostname_prefix, hostname_suffix, dup_action=num_suffixが
      # 対象。
      t.boolean :is_dn_orig, null: false, default: true

      # 有効期間(begin_date～end_date)
      # 日時(datetime)ではなく日付(date)なので注意。
      # 両方ともnilなら期限なし
      t.date :begin_date, default: nil
      t.date :end_date, default: nil

      t.timestamps
    end

    # 登録キー毎のカテゴリ
    # 少なくとも一つのシステムカテゴリを登録すること。
    # また、登録時にカスタムカテゴリが無かったら、登録に失敗する。
    create_table :sidfmd_regkey_cats do |t|
      t.references :sidfmd_regkey, null: false, index: true
      # カテゴリID
      t.references :sidfmd_category, null: false
    end
  end
end
